#include <stddef.h>                     // Defines NULL
#include <stdbool.h>                    // Defines true
#include "stm32h5xx.h"
#include "stm32h5xx_hal_pcd.h"
#include "usbd_def.h"
#include "ringbuf3b.h"

#define LAY_SNR		"LE0050"

#define DEBUG_CONSOLE_ON	0

#define STATIC static
#define PCprintf(...) 

#if DEBUG_CONSOLE_ON == 0
#define printf(...)
#endif

#ifdef INIMP_2K5
#define INIMP_10K
#endif

#define INDC_DIGIT_V		0.00432

#define SYS_CLK			240000000
#define SYS_CLK_COUNT_P_MS	10	/* 100us */

#define CONSOLE_BAUD_RATE	115200

#define PIN_OPVEN_ON		GPIOB->BSRR = GPIO_BSRR_BS0
#define PIN_OPVEN_STAT		(GPIOB->ODR & 1)

#define PIN_5VEN_ON		GPIOA->BSRR = GPIO_BSRR_BS2

#define TRG_IO_L		GPIOA->BSRR = GPIO_BSRR_BR3
#define TRG_IO_H		GPIOA->BSRR = GPIO_BSRR_BS3
#define TRG_IO_BNR		3

#define TEST_PIN_L		GPIOA->BSRR = GPIO_BSRR_BR5
#define TEST_PIN_H		GPIOA->BSRR = GPIO_BSRR_BS5

#define TRG_O_EN_L		GPIOA->BSRR = GPIO_BSRR_BR6
#define TRG_O_EN_H		GPIOA->BSRR = GPIO_BSRR_BS6

#define LED_RED_ON		GPIOB->BSRR = GPIO_BSRR_BR1
#define LED_RED_OFF		GPIOB->BSRR = GPIO_BSRR_BS1
#define LED_RED_STAT		(GPIOB->ODR & (1<<1))

#define LED_GREEN_ON		GPIOB->BSRR = GPIO_BSRR_BR2
#define LED_GREEN_OFF		GPIOB->BSRR = GPIO_BSRR_BS2
#define LED_GREEN_STAT		(GPIOB->ODR & (1<<2))

#define ADC_FILT1_L		GPIOB->BSRR = GPIO_BSRR_BR10
#define ADC_FILT1_H		GPIOB->BSRR = GPIO_BSRR_BS10
#define ADC_FILT1_STAT		(GPIOB->ODR & (1<<10))

#define ADC_RESET_L		GPIOB->BSRR = GPIO_BSRR_BR12
#define ADC_RESET_H		GPIOB->BSRR = GPIO_BSRR_BS12

#define ADC_CS_L		GPIOB->BSRR = GPIO_BSRR_BR13
#define ADC_CS_H		GPIOB->BSRR = GPIO_BSRR_BS13

#define ADC_K2_OFF		GPIOB->BSRR = GPIO_BSRR_BR8
#define ADC_K2_ON		GPIOB->BSRR = GPIO_BSRR_BS8
#define ADC_K2_STAT		(GPIOB->ODR & (1<<8))

#define ADC_K3_OFF		GPIOC->BSRR = GPIO_BSRR_BR13
#define ADC_K3_ON		GPIOC->BSRR = GPIO_BSRR_BS13
#define ADC_K3_STAT		(GPIOC->ODR & (1<<13))

#define ADC_K4_OFF		GPIOC->BSRR = GPIO_BSRR_BR14
#define ADC_K4_ON		GPIOC->BSRR = GPIO_BSRR_BS14
#define ADC_K4_STAT		(GPIOC->ODR & (1<<14))


#define INP_ATT_OFF		GPIOB->BSRR = GPIO_BSRR_BR5
#define INP_ATT_ON		GPIOB->BSRR = GPIO_BSRR_BS5
#define INP_ATT_STAT		(GPIOB->ODR & (1<<5))

#define INP_DC_OFF		GPIOC->BSRR = GPIO_BSRR_BR15
#define INP_DC_ON		GPIOC->BSRR = GPIO_BSRR_BS15
#define INP_DC_STAT		(GPIOC->ODR & (1<<15))

#define INP_CPEN_OFF		GPIOB->BSRR = GPIO_BSRR_BR4
#define INP_CPEN_ON		GPIOB->BSRR = GPIO_BSRR_BS4
#define INP_CPEN_STAT		(GPIOB->ODR & (1<<4))

#define SPI1_RDY_PIN		10
#define SPI1_RDY_STAT		(GPIOA->IDR & (1<<SPI1_RDY_PIN))

typedef union {
  uint32_t u;
  int32_t  i;
  float  f;
  uint16_t u16[2];
  int16_t  i16[2];
  uint8_t u8[4];
  int8_t  i8[4];
  char  c[4];
} Txc32;

extern __IO uint32_t TimerCount;
extern __IO uint32_t TimerCount_ms;
extern uint32_t LoopCount;

void HAL_Delay(uint32_t wait_ms);

extern float InDC;
extern float InDC_DigitV;
extern float UsbV;

#if 0
static __inline__ int TestFloatOk(float *f) {
 Txc32 xc32;
 uint16_t e;
		xc32.f = *f;
		e = (xc32.u16[1] >> 7) & 0xff;
		if ((e == 0xff) || (!e && xc32.u)) return 1;
		return 0;
}
#else
int TestFloatHasError(float *f);
#endif
/*
#define TestFloatOk(pf) {Txc32 zzxc32; uint16_t e; \
		zzxc32.f = *pf; \
		e = (zzxc32.u16[1] >> 7) & 0xff; \
		if ((e == 0xff) || (!e && zzxc32.u)) return 1; \
		return 0; \
}
*/

/* ----------------------- main Exec Cmd -------------------------------- */
#define EXEC_SAVE_CAL_DATA	1
#define EXEC_CLR_BUF_DATA	(1<<1)

extern uint16_t ExecCmdBef;

/* ----------------------- USB ------------------------------------------ */

#define USB_USE_HAL	0
#define USB_SINGLE_FS	1

extern PCD_HandleTypeDef hpcd;
extern USBD_HandleTypeDef USB_Dev;

//#define USE_USB_IRQ_SOFM		USB_CNTR_SOFM
#define USE_USB_IRQ_PMAOVR		0
#define USE_USB_IRQ_ERR			0
#define USE_USB_IRQ_SOF			0
#define USE_USB_IRQ_ESOF		0
#define USE_USB_IRQ_L1REQ		0

#define USE_USB_IRQ	USB_CNTR_CTRM  | USB_CNTR_WKUPM | \
			USB_CNTR_SUSPM | USE_USB_IRQ_ERR | \
			USE_USB_IRQ_SOF | USE_USB_IRQ_ESOF | \
			USB_CNTR_RESETM | USE_USB_IRQ_L1REQ

/* ----------------------- FALSH ---------------------------------------- */
#define FLASH_DATA_SECTOR		((FLASH_SECTOR_NB * 2) - 1)
#define FLASH_DATA_ADR			(FLASH_BASE + FLASH_SECTOR_SIZE * FLASH_DATA_SECTOR)

#define FLASH_MAX_USER_SEC		16
#define FLASH_MAX_USER_SEC_SIZE		(FLASH_SECTOR_SIZE / FLASH_MAX_USER_SEC)

#ifdef INIMP_10K
#define FLASH_DATA_VER			100
#else
#define FLASH_DATA_VER			0
#endif
#define FLASH_DATA_KENN(a) (uint16_t)(a | (~a << 8))

typedef struct {
  uint16_t kenn;
  uint16_t ver;

  float   TempOffs;
  float   AdcVpC;		/* V/C */
  int32_t nc32_1;
  int32_t nc32_2;
  int32_t nc32_3;
  int32_t nc32_4;
  int32_t nc32_5;

  int16_t nc0;
#ifdef INIMP_10K
  int16_t V5000mV_Offs;
  int16_t V2000mV_Offs;
#else
  int16_t nc1;
  int16_t nc2;
#endif
  int16_t V1000mV_Offs;
  int16_t V500mV_Offs;
  int16_t V200mV_Offs;
  int16_t V100mV_Offs;
  int16_t V50mV_Offs;
  int16_t V20mV_Offs;
  int16_t nc3;

  float   fc0;
#ifdef INIMP_10K
  float   V5000mV_Mul;
  float   V2000mV_Mul;
#else
  float   fc1;
  float   fc2;
#endif
  float   V1000mV_Mul;
  float   V500mV_Mul;
  float   V200mV_Mul;
  float   V100mV_Mul;
  float   V50mV_Mul;
  float   V20mV_Mul;
  float   fc3;

  int16_t K2_TV;
  int16_t K3_TV;
  int16_t K4_TV;

#ifdef INIMP_10K
  int16_t nct16_4;
#else
  int16_t OffsAC;
#endif

  int16_t  InDC_Offset;
  uint16_t InDC_DigitV;
  int16_t nct16_7;
  int16_t nct16_8;
  int16_t nct16_9;
  int16_t nct16_10;

} TFlashData;

extern uint8_t FlashBuffer[sizeof(TFlashData) + 16];

/* Test */
extern uint32_t CmdTestBef;
extern float CmdTestFloat, CmdTestFloatResult;

extern RingBuffer3B rb3b;
