
#ifndef __OSZI_H__
#define __OSZI_H__

#define USE_OSZI_TDIV_MOD	1

#include "ADS127L01.h"


#define OSZI_BANDWIDTH_OFF	0
#define OSZI_BANDWIDTH_AUTO	1

#define OSZI_TRIGFKT_ON		1
#define OSZI_TRIGFKT_SINGLE	(1<<1)
#define OSZI_TRIGFKT_ON_SINGLE	(OSZI_TRIGFKT_ON | OSZI_TRIGFKT_SINGLE)
#define OSZI_TRIGFKT_FREERUN	4
#define OSZI_TRIGFKT_EXT	8
#define OSZI_TRIGFKT_MASK	0x0f
#define OSZI_TRIGFKT_NEG	(1<<4)

#define USE_OSZI_VDIV_TAB	1

typedef struct {
  uint32_t DataCount;
  uint8_t InCoup;	/* 0=DC 1=AC */
  uint8_t TrigFkt;
  uint8_t V_DivCount;
  uint8_t T_DivCount;

  uint8_t BandWidth;	/* 0=off 1=auto */
  uint8_t nc1;
  uint8_t nc2;
#if USE_OSZI_VDIV_TAB
  uint8_t tab_idx_V_Div;
#else
  uint8_t nc3;
#endif

  uint32_t TrigToll;
  int32_t TrigHV;
  int32_t TrigLV;

  int32_t  GainOffset;
  int32_t  TempOffset;
#ifndef INIMP_10K
  int32_t  AC_Offset;
#endif

//  int32_t  DacOffset;  /* Data + DacOffset = Value */
//  int32_t Data0_Offset; /* Data + Data0_Offset = Value */

  float TrigV;
  float ExtTrigV;
  float TrigT;
  float TrigTimeProz;
  float V_Div;
  float T_Div;
  float Samps_pSec;
  float OverSampF;

  float dac_V;
  float dac_digit_V;
  float dac_Vmul;
  float dac_Vmul_korr;


} TOsziConfig;

extern TOsziConfig OsziConfig;

void Oszi_ConfigInit();

typedef struct {
  char name[20];
  uint16_t head_size;
} __attribute__((packed)) TOsziFileHead;

typedef struct {
  uint16_t struct_size;	
  uint16_t ncw0;	
  uint32_t DataCount;
  uint8_t version;
  uint8_t dtype;	/* 0=Byte 1=word 2=3Byte 3=4Byte 8=float bit7=1==Unsign */
  uint8_t samp_bit;
  uint8_t Channel;
  uint8_t V_DivCount;
  uint8_t T_DivCount;
  uint8_t nc0;
  uint8_t nc1;
  char name[16];
  int32_t Data0_Offset; /* Data + Data0_Offset = Value */
  float GainMul;	/* 1 Digit Volt */
  float samp_t;
  float start_t;
  float V_Div;
  float T_Div;
  uint16_t FrameNum;
  uint16_t ncw1;

} __attribute__((packed)) TOsziDataHead;

typedef struct {
  TOsziFileHead fh;
  TOsziDataHead oh;
} __attribute__((packed)) TOsziHead;


void Oszi_DataHeadInit();
void Oszi_KorrTempAdcOffset();

bool Oszi_SetMDataCount(const char *s);
uint32_t Oszi_GetMDataCount();

bool Oszi_SetT_Div(const char *s);
float Oszi_GetT_Div();

bool Oszi_SetV_DivStr(const char *s);
bool Oszi_SetV_Div();
float Oszi_GetV_Div();

bool Oszi_SetTrigSource(const char *s);
void Oszi_GetTrigSource();
bool Oszi_SetTrigTOffset(const char *s);
//void Oszi_GetTrigTOffset();
bool Oszi_SetTrigType(const char *s);
void Oszi_GetTrigType();
bool Oszi_SetTrigToll(const char *s);
void Oszi_GetTrigToll();
bool Oszi_SetTrigV(const char *s);
void Oszi_GetTrigV();
bool Oszi_RunSingle();

bool Oszi_SetBandWidthLimit(const char *pc);
bool Oszi_SetInCoup(const char *pc);
bool Oszi_SetWideBand(const char *pc);

void Oszi_DmmV_Conv(int32_t AnlVal);
float Oszi_Get_DmmV();

bool Oszi_TestVal(const char *pc);
bool Oszi_Test_IRR_Val(const char *pc);

int32_t Oszi_GetBufferStat();

void Oszi_ExtTrigOutEn(bool en);
void Oszi_ExtTrigSetV(float v);


#endif	/* __OSZI_H__ */
