
#include "main.h"
#include "libh523.h"
#include "usbd_tmc_interface.h"
#include <stdlib.h>
#include <stdio.h>

__attribute__((aligned(16))) 
void USB_DRD_FS_IRQHandler(void) {

  PCD_EPTypeDef *ep;
  uint16_t count, wIstr, wEPVal;
  uint8_t epindex;

	wIstr = USB_DRD_FS->ISTR;

	if (wIstr & USB_ISTR_CTR) {

	    count = 0;
	    /* stay in loop while pending interrupts */
//	    while ((USB_DRD_FS->ISTR & USB_ISTR_CTR) != 0U) {
//	      wIstr = (uint16_t)USB_DRD_FS->ISTR;

	      /* extract highest priority endpoint number */
	      epindex = (uint8_t)(wIstr & USB_ISTR_IDN);
	      if (epindex == 0U) {
		  /* DIR = 0 */
		/* Decode and service control endpoint interrupt */
		/* DIR bit = origin of the interrupt */
		if ((wIstr & USB_ISTR_DIR) == 0U) {
		  /* DIR = 0 => IN  int */
		  /* DIR = 0 implies that (EP_CTR_TX = 1) always */
		  USB_DRD_CLEAR_TX_CHEP_CTR(USB_DRD_FS, 0);
		  ep = &hpcd.IN_ep[0];

		  ep->xfer_count = USB_DRD_GET_CHEP_TX_CNT(USB_DRD_FS, ep->num);
		  ep->xfer_buff += ep->xfer_count;

		  /* TX COMPLETE */
		  HAL_PCD_DataInStageCallback(&hpcd, 0U);

		  if ((hpcd.USB_Address > 0U) && (ep->xfer_len == 0U)) {
		    USB_DRD_FS->DADDR = ((uint16_t)hpcd.USB_Address | USB_DADDR_EF);
		    hpcd.USB_Address = 0U;
		  }

		} else {
		  /* DIR = 1 */
		  /* DIR = 1 & CTR_RX => SETUP or OUT int */
		  /* DIR = 1 & (CTR_TX | CTR_RX) => 2 int pending */
		  ep = &hpcd.OUT_ep[0];
		  wEPVal = (uint16_t)USB_DRD_GET_CHEP(USB_DRD_FS, 0);

		  if ((wEPVal & USB_EP_SETUP) != 0U) {
		    /* Get SETUP Packet */
		    ep->xfer_count = PCD_GET_EP_RX_CNT(USB_DRD_FS, ep->num);

		    USB_ReadPMA(USB_DRD_FS, (uint8_t *)hpcd.Setup,
				ep->pmaadress, (uint16_t)ep->xfer_count);

		    /* SETUP bit kept frozen while CTR_RX = 1 */
		    USB_DRD_CLEAR_RX_CHEP_CTR(USB_DRD_FS, 0);

		    /* Process SETUP Packet*/
		    HAL_PCD_SetupStageCallback(&hpcd);

		  } else if ((wEPVal & USB_EP_VTRX) != 0U) {
		    USB_DRD_CLEAR_RX_CHEP_CTR(USB_DRD_FS, 0);

		    /* Get Control Data OUT Packet */
		    ep->xfer_count = PCD_GET_EP_RX_CNT(USB_DRD_FS, ep->num);

		    if ((ep->xfer_count != 0U) && (ep->xfer_buff != 0U)) {
		      USB_ReadPMA(USB_DRD_FS, ep->xfer_buff,
				  ep->pmaadress, (uint16_t)ep->xfer_count);

		      ep->xfer_buff += ep->xfer_count;
		      /* Process Control Data OUT Packet */
		      HAL_PCD_DataOutStageCallback(&hpcd, 0U);
		    }

		    wEPVal = (uint16_t)USB_DRD_GET_CHEP(USB_DRD_FS, 0);

		    if (((wEPVal & USB_EP_SETUP) == 0U) && ((wEPVal & USB_EP_RX_STRX) != USB_EP_RX_VALID)) {
		      USB_DRD_SET_CHEP_RX_CNT(USB_DRD_FS, 0, ep->maxpacket);
		      USB_DRD_SET_CHEP_RX_STATUS(USB_DRD_FS, 0, USB_EP_RX_VALID);
		    }
		  }
		}
	      } else {

		/* Decode and service non control endpoints interrupt */
		/* process related endpoint register */

		wEPVal = (uint16_t)USB_DRD_GET_CHEP(USB_DRD_FS, epindex);

		if ((wEPVal & USB_EP_VTRX) != 0U) {

		  /* clear int flag */
		  USB_DRD_CLEAR_RX_CHEP_CTR(USB_DRD_FS, epindex);
//		  USB_DRD_SET_CHEP(USB_DRD_FS, epindex, (wEPVal & (0xFFFF7FFFU & USB_CHEP_REG_MASK)) | USB_CHEP_VTTX);
		  ep = &hpcd.OUT_ep[epindex & MAX_USBD_EP_IO_MASK];
#if 1	/* time 1Byte=540ns .. 64Byte=540ns  */
  uint32_t *pdwVal;
// TEST_PIN_H;
		  pdwVal = (uint32_t *)(USB_DRD_PMAADDR + (uint32_t)ep->pmaadress);
		  USB_Move64Byte((uint32_t*)ep->xfer_buff, pdwVal);
// TEST_PIN_L;
		  count = (uint16_t)USB_DRD_GET_CHEP_RX_CNT(USB_DRD_FS, ep->num);
#else	/* time 1Byte=575ns .. 64Byte=1240ns  */
		/* OUT Single Buffering */
		/* PCD_GET_EP_RX_CNT() use: few cycles for RX PMA descriptor to update */
		  count = (uint16_t)PCD_GET_EP_RX_CNT(USB_DRD_FS, ep->num);
		  if (count != 0U) {
		    USB_ReadPMA(USB_DRD_FS, ep->xfer_buff, ep->pmaadress, count);
		  }
#endif
		  /* multi-packet on the NON control OUT endpoint */
		  ep->xfer_count += count;
		  ep->xfer_buff += count;
//		  if ((ep->xfer_len == 0U) || (count < ep->maxpacket)) {
		    /* RX COMPLETE */
		    HAL_PCD_DataOutStageCallback(&hpcd, ep->num);
//		  } else {
//		    (void)USB_EPStartXfer(USB_DRD_FS, ep);
//		  }
		}

		if ((wEPVal & USB_EP_VTTX) != 0U) {
		  ep = &hpcd.IN_ep[epindex & MAX_USBD_EP_IO_MASK];
		  /* clear int flag */
		  USB_DRD_CLEAR_TX_CHEP_CTR(USB_DRD_FS, epindex);

		  if (ep->type == EP_TYPE_ISOC) {
		    ep->xfer_len = 0U;
		    /* TX COMPLETE */
		    HAL_PCD_DataInStageCallback(&hpcd, ep->num);

		  } else {
		    /* Manage Single Buffer Transaction */
		    if ((wEPVal & USB_EP_KIND) == 0U) {

		      if (ep->xfer_len > USB_FS_MAX_PACKET_SIZE) {
			ep->xfer_len -= USB_FS_MAX_PACKET_SIZE;
		      } else {
			ep->xfer_len = 0U;
		      }

		      /* Zero Length Packet? */
		      if (ep->xfer_len == 0U) {
			/* TX COMPLETE */
			HAL_PCD_DataInStageCallback(&hpcd, ep->num);
		      } else {
			/* Transfer is not yet Done */
			ep->xfer_buff += USB_FS_MAX_PACKET_SIZE;
			ep->xfer_count += USB_FS_MAX_PACKET_SIZE;
			TMC_TransmitNext(&hpcd, ep);
// TEST_PIN_H;
#if 1
			USB_EPStartXfer(USB_DRD_FS, ep);
#else
  uint32_t len;
			if (ep->xfer_len > USB_FS_MAX_PACKET_SIZE) {
			  len = USB_FS_MAX_PACKET_SIZE;
			} else {
			  len = ep->xfer_len;
			}

#if 1
			USB_WritePMA(USB_DRD_FS, ep->xfer_buff, ep->pmaadress, (uint16_t)len);
#else
  uint32_t i_count, NbWords, *pdwVal, *pBuf;

  NbWords = (len + 3) >> 2;
  /* Get the PMA Buffer pointer */
  pdwVal = (uint32_t *)(USB_DRD_PMAADDR + (uint32_t)ep->pmaadress);
  pBuf = (uint32_t *)ep->xfer_buff;

  for (i_count=NbWords; i_count; i_count--) {
      *pdwVal++ = *pBuf++;
  }
#endif
			USB_DRD_SET_CHEP_TX_CNT(USB_DRD_FS, ep->num, len);
			USB_DRD_SET_CHEP_TX_STATUS(USB_DRD_FS, ep->num, USB_EP_TX_VALID);
#endif
// TEST_PIN_L;


		      }
		    }
		  }
		}
	      }
//	    }
#if USE_USB_IRQ_SOFM
	} else if (wIstr & USB_ISTR_SOF) {
	    USB_DRD_FS->ISTR = ~USB_ISTR_SOF & 0x0000FFFF;
	    HAL_PCD_SOFCallback(&hpcd);
#endif

#if USE_USB_IRQ_ESOFM
	} else if (wIstr & USB_ISTR_ESOF) {
	    USB_DRD_FS->ISTR = ~USB_ISTR_ESOF & 0x0000FFFF;
#endif

	} else if (wIstr & USB_ISTR_RESET) {
	    USB_DRD_FS->ISTR = ~USB_ISTR_RESET & 0x0000FFFF;
	    HAL_PCD_ResetCallback(&hpcd);

#if USE_USB_IRQ_PMAOVR
	} else if (wIstr & USB_ISTR_PMAOVR) {
	    USB_DRD_FS->ISTR = ~USB_ISTR_PMAOVR & 0x0000FFFF;
#endif

#if USE_USB_IRQ_ERR
	} else if (wIstr & USB_ISTR_ERR) {
	    USB_DRD_FS->ISTR = ~USB_ISTR_ERR & 0x0000FFFF;
#endif
	} else if (wIstr & USB_ISTR_WKUP) {
	    USB_DRD_FS->CNTR &= ~(USB_CNTR_SUSPRDY);
	    USB_DRD_FS->CNTR &= ~(USB_CNTR_SUSPEN);
#if USE_USB_IRQ_L1REQ
	    if (hpcd.LPM_State == LPM_L1) {
	      hpcd.LPM_State = LPM_L0;
	      HAL_PCDEx_LPM_Callback(&hpcd, PCD_LPM_L0_ACTIVE);
	    }
#endif
	    HAL_PCD_ResumeCallback(&hpcd);
	    USB_DRD_FS->ISTR = ~USB_ISTR_WKUP & 0x0000FFFF;

	} else if (wIstr & USB_ISTR_SUSP) {
	    USB_DRD_FS->CNTR |= USB_CNTR_SUSPEN;
	    /* clear of the ISTR bit must be done after setting of CNTR_FSUSP */
	    USB_DRD_FS->ISTR = ~USB_ISTR_SUSP & 0x0000FFFF;
	    hpcd.Instance->CNTR |= USB_CNTR_SUSPRDY;
	    HAL_PCD_SuspendCallback(&hpcd);
#if USE_USB_IRQ_L1REQ
	} else if (wIstr & USB_ISTR_L1REQ) {
	    USB_DRD_FS->ISTR = ~USB_ISTR_L1REQ & 0x0000FFFF;
	    if (hpcd.LPM_State == LPM_L0) {
		/* Force suspend and low-power mode before going to L1 state*/
		USB_DRD_FS->CNTR |= USB_CNTR_SUSPRDY;
		USB_DRD_FS->CNTR |= USB_CNTR_SUSPEN;

		hpcd.LPM_State = LPM_L1;
		hpcd.BESL = ((uint32_t)USB_DRD_FS->LPMCSR & USB_LPMCSR_BESL) >> 2;
		HAL_PCDEx_LPM_Callback(&hpcd, PCD_LPM_L1_ACTIVE);
	    } else {
		HAL_PCD_SuspendCallback(&hpcd);
	    }
#endif
	} else {

#if DEBUG_CONSOLE_ON
 printf("%04x\r\n", (int)wIstr);
#endif


//	    HAL_PCD_IRQHandler(&hpcd);
	}
// printf("%04x\r\n", (int)USB_DRD_FS->ISTR);
}

__weak void HAL_PCD_ResetCallback(PCD_HandleTypeDef *hpcd) {
}

#if USE_USB_IRQ_L1REQ
__weak void HAL_PCDEx_LPM_Callback(PCD_HandleTypeDef *hpcd, PCD_LPM_MsgTypeDef msg) {
}
#endif
