
#include "main.h"
#include "scpi.h"
#include "oszi.h"

static TAds127l01 ex_adc1;

void Ads127l01_Init() {

/* ADC_CLK = 16MHz    pll=480 / 5 = 96 MHz  96 / 6 = 16 MHz   */
	MODIFY_REG(RCC->CFGR1, 
			RCC_CFGR1_MCO1PRE | RCC_CFGR1_MCO1SEL, 
			(6 << RCC_CFGR1_MCO1PRE_Pos) | (3 << RCC_CFGR1_MCO1SEL_Pos));

	MODIFY_REG(GPIOA->MODER, GPIO_MODER_MODE8, GPIO_MODER_MODE8_1);
	MODIFY_REG(GPIOA->AFR[1], 
			GPIO_AFRH_AFSEL8,
			(0 << GPIO_AFRH_AFSEL8_Pos));

	ADC_FILT1_H;

/* SPI2 Configure CLK=PA9 IN=PB14 OUT=PA15 */
	RCC->APB1LENR |= RCC_APB1LENR_SPI2EN;
	MODIFY_REG(GPIOA->MODER, 
		GPIO_MODER_MODE9,
		GPIO_MODER_MODE9_1);
	MODIFY_REG(GPIOB->MODER, 
		GPIO_MODER_MODE14   | GPIO_MODER_MODE15,
		GPIO_MODER_MODE14_1 | GPIO_MODER_MODE15_1);

	GPIOB->OSPEEDR |= GPIO_OSPEEDR_OSPEED14_0 | GPIO_OSPEEDR_OSPEED15_0;

	MODIFY_REG(GPIOA->AFR[1], 
			GPIO_AFRH_AFSEL9,
			(5 << GPIO_AFRH_AFSEL9_Pos));
	MODIFY_REG(GPIOB->AFR[1], 
			GPIO_AFRH_AFSEL14 | GPIO_AFRH_AFSEL15,
			(5 << GPIO_AFRH_AFSEL14_Pos) |
			(5 << GPIO_AFRH_AFSEL15_Pos));

	/* SPI CLK=24  96/4  32Bit */
	SPI2->CFG1 = (1 << SPI_CFG1_MBR_Pos) | 31 | (7 << SPI_CFG1_CRCSIZE_Pos);
#if 1
	NVIC_EnableIRQ(SPI2_IRQn);
	SPI2->IER = SPI_IER_RXPIE;
	NVIC_SetPriority(SPI2_IRQn, 4);
#endif
	SPI2->CR1 = SPI_CR1_SSI;
	SPI2->CFG2 = SPI_CFG2_SSOM | SPI_CFG2_SSM | SPI_CFG2_MASTER | SPI_CFG2_CPHA;
	SPI2->CR1 = SPI_CR1_SSI | SPI_CR1_SPE;
	SPI2->CR1 = SPI_CR1_SPE | SPI_CR1_SSI | SPI_CR1_CSTART;


/* RDY PA10 */
	CLEAR_BIT(GPIOA->MODER, GPIO_MODER_MODE10);
	GPIOA->PUPDR |= GPIO_PUPDR_PUPD10_0;

//	NVIC_EnableIRQ(EXTI10_IRQn);
	NVIC_SetPriority(EXTI10_IRQn, 4);
/* EXTICR3  0x00: PA10 pin */
//	EXTI->EXTICR[3] = (0 << EXTI_EXTICR3_EXTI10_Pos);
	EXTI->FTSR1 |= (1<<SPI1_RDY_PIN);
	EXTI->IMR1  |= (1<<SPI1_RDY_PIN);

	HAL_Delay(1);
	ADC_RESET_H;

//	SPI2->TXDR = 0x12000000;
}

void Ads127l01_Start() {
	ex_adc1.OverSampCount = 0;
	NVIC_EnableIRQ(EXTI10_IRQn);
}

void Ads127l01_Stop() {
	NVIC_DisableIRQ(EXTI10_IRQn);
}

//void Ads127l01_Handle() {
//}

#ifndef USE_ADS127_IT_ASM

void Ads127l01_SetOverSampDiv(int32_t OverSampDiv1, int32_t OverSampDiv2) {
	ex_adc1.OverSampDiv = OverSampDiv1;
	ex_adc1.OverSampCount = 0;
	ex_adc1.OverSampSum = 0;

	ex_adc1.OverSampDiv2 = OverSampDiv2;
	ex_adc1.OverSampCount2 = 0;
	ex_adc1.OverSampSum2 = 0;
}

void Ads127l01_SetBandWidth(bool bw_auto) {
	if (bw_auto) {
	    ex_adc1.Fkt |= ADSF_BAND_WIDTH_AUTO;
	} else {
	    ex_adc1.Fkt &= ~ADSF_BAND_WIDTH_AUTO;
	}
}

void Ads127l01_SetTrigger(int32_t TrigHV, int32_t TrigLV, uint16_t TrigMod) {
	ex_adc1.TrigHV = TrigHV;
	ex_adc1.TrigLV = TrigLV;
	ex_adc1.TrigMod = TrigMod;
	ex_adc1.TrigStat = 0;
}

void Ads127l01_SetIRR(uint32_t IRR_InMul, uint32_t fkt) {
	ex_adc1.IRR_InMul = 0;
	if (!IRR_InMul) return;
//	if (IRR_InMul > 16) IRR_InMul = 16;
//	IRR_InMul--;
//	ex_adc1.IRR_InMul = 0x40000000 >> IRR_InMul;

	ex_adc1.IRR_InMul = IRR_InMul;
	ex_adc1.IRR_yMUL = (uint32_t)0x80000000 - 
			  (uint32_t)(ex_adc1.IRR_InMul << 1);
}

bool Ads127l01_GetAnlVal(int32_t *AnlVal) {
	if (ex_adc1.Anl_Stat) return false;
	ex_adc1.Anl_Stat = 1;
	*AnlVal = ex_adc1.FValue;
}

__attribute__((aligned(16))) 
void SPI2_IRQHandler(void) {
  int32_t din;
	din = SPI2->RXDR << 8;
	ex_adc1.spi_data = din >> 8;
	ADC_CS_H;
}
#endif

#ifndef USE_ADS127_IT_ASM
__attribute__((aligned(16))) 
void EXTI10_IRQHandler(void) {
  int32_t din;
  int64_t res, r2;
  Tx64 x64;
  TAds127l01 *padc;

// TEST_PIN_H;
	if (EXTI->FPR1 & (1<<SPI1_RDY_PIN)) {
	    EXTI->FPR1 = 1<<SPI1_RDY_PIN;
	    ADC_CS_L;
	    SPI2->TXDR = 0x12000000;
	    padc = &ex_adc1;
	    din = padc->spi_data;

#if 0
	    if (padc->IRR_InMul > 0) {

		r2  = (int64_t)din * (int64_t)padc->IRR_InMul;
		res = r2 + padc->IRR_X;
		padc->IRR_X = r2;

		x64.u32[0] = padc->IRR_Y.u32[0];
		x64.u32[1] = 0;
		x64.i64 *= padc->IRR_yMUL;

	//	if (x64.u32[0] & 0x80000000) x64.u32[1]++;
		x64.u32[0] = x64.u32[1];
		x64.u32[1] = 0;

		r2 = (int64_t)padc->IRR_Y.i32[1] * (int64_t)padc->IRR_yMUL;
		r2 += x64.i64;

		padc->IRR_Y.i64 = r2 + res;
		padc->IRR_Y.i64 <<= 1;

		din = padc->IRR_Y.i32[1];
	    }
#endif

#if 0
	    padc->F256sum += din;
	    padc->F256count++;
	    if ((padc->F256count & 0x003f) == 0) {
		din = padc->F256sum >> 6;
		padc->F256sum = 0;

		padc->F40sum += din;
		padc->F40count++;
 /* 500000 / 50Hz / 64 = 156.25 */
 /* 500000 / 60Hz / 64 = 130.21 */
		if (padc->F40count >= 156) {   /* 130=60Hz */
		    padc->FValue = padc->F40sum / padc->F40count;
		    padc->F40sum = 0;
		    padc->F40count = 0;
		}
	    }
	    din = padc->FValue;
#endif

#if 0
  int32_t res, zw;
	zw = (din * 8);
	res = zw + padc->Anl_IRR_X;
	padc->Anl_IRR_X = zw;
	padc->Anl_IRR_Y = (res + (padc->Anl_IRR_Y * 15)) >> 4;
	din = padc->Anl_IRR_Y >> 4;

#endif

#if 1
  int32_t res, zw;
  int64_t i64;
	zw = din;
	res = zw + padc->Anl_IRR_X;
	padc->Anl_IRR_X = zw;
	x64.i64 = ((int64_t)padc->Anl_IRR_Y * (int64_t)127) >> 7;
	padc->Anl_IRR_Y = res + x64.i32[0];
	din = padc->Anl_IRR_Y >> 8;
#if 1
	res = din + padc->Anl_IRR_X2;
	padc->Anl_IRR_X2 = din;
	x64.i64 = ((int64_t)padc->Anl_IRR_Y2 * (int64_t)127) >> 7;
	padc->Anl_IRR_Y2 = res + x64.i32[0];
	din = padc->Anl_IRR_Y2 >> 8;
#endif
#endif



#if 1
	    padc->F256sum += din;
	    padc->F256count++;
	    if (padc->F256count >= 100) {
		padc->FValue = padc->F256sum / padc->F256count;
		padc->F256count = 0;
		padc->F256sum = 0;
		padc->Anl_Stat = 0;
	    }
	    din = padc->FValue;
#endif

#if 0
	    padc->F256count++;
	    if (padc->F256count & 0x00ff) {
		padc->F256sum += din;
		din = padc->FValue;

	    } else {
		padc->F256sum += din;
		padc->FValue = padc->F256sum >> 8;
		padc->F256sum = 0;
		din = padc->FValue;
	    }
#endif

#if 1
/* ---------------------------------------------------------------------- */
	    if (padc->OverSampDiv > 1) {
		padc->OverSampSum += din;
		padc->OverSampCount++;
		if (padc->OverSampCount < padc->OverSampDiv) goto Lend;
		padc->OverSampCount = 0;
		if (ex_adc1.Fkt & ADSF_BAND_WIDTH_AUTO) {
		    din = padc->OverSampSum / padc->OverSampDiv;
		    padc->OverSampSum = 0;
		}
	    }

	    if (padc->OverSampDiv2 > 1) {
		padc->OverSampSum2 += din;
		padc->OverSampCount2++;
		if (padc->OverSampCount2 < padc->OverSampDiv2) goto Lend;
		padc->OverSampCount2 = 0;
		if (ex_adc1.Fkt & ADSF_BAND_WIDTH_AUTO) {
		    din = padc->OverSampSum2 / padc->OverSampDiv2;
		    padc->OverSampSum2 = 0;
		}
	    }
#endif

#if 1
	    if (din > padc->TrigHV) {
		if ((padc->TrigStat & TRIG_ST_H) == 0) {
		    padc->TrigStat |= TRIG_ST_H;
		    if (padc->TrigMod & TRIG_LH) {
// TEST_PIN_H;
// PCprintf("H Trig LH %d\n", din);
			RingBuf3B_SetTrig(&Scpi.p3);
		    }
		}

	    } else if (din < padc->TrigLV) {
		if (padc->TrigStat & TRIG_ST_H) {
		    padc->TrigStat &= ~TRIG_ST_H;
		    if (padc->TrigMod & TRIG_HL) {
// PCprintf("L Trig HL %d\n", din);
			RingBuf3B_SetTrig(&Scpi.p3);
		    }
		}
	    }
#endif
	    RingBuf3B_Add(&Scpi.p3, (uint8_t*)&din);
	}
Lend:
// TEST_PIN_L;
	return;
}

#endif  /* USE_ADS127_IT_ASM */
