
#include "main.h"
#include <stdio.h>
#include "libh523.h"

#define INJECTED	1

/* The clock is common for all the ADCs. */
#define ADC_CLOCK_ASYNC_DIV1            0
#define ADC_CLOCK_ASYNC_DIV2            (0x01 << ADC_CCR_PRESC_Pos)
#define ADC_CLOCK_ASYNC_DIV4            (0x02 << ADC_CCR_PRESC_Pos)
#define ADC_CLOCK_ASYNC_DIV6            (0x03 << ADC_CCR_PRESC_Pos)
#define ADC_CLOCK_ASYNC_DIV8            (0x04 << ADC_CCR_PRESC_Pos)
#define ADC_CLOCK_ASYNC_DIV10           (0x05 << ADC_CCR_PRESC_Pos)
#define ADC_CLOCK_ASYNC_DIV12           (0x06 << ADC_CCR_PRESC_Pos)
#define ADC_CLOCK_ASYNC_DIV16           (0x07 << ADC_CCR_PRESC_Pos)
#define ADC_CLOCK_ASYNC_DIV32           (0x08 << ADC_CCR_PRESC_Pos)
#define ADC_CLOCK_ASYNC_DIV64           (0x09 << ADC_CCR_PRESC_Pos)
#define ADC_CLOCK_ASYNC_DIV128          (0x0A << ADC_CCR_PRESC_Pos)
#define ADC_CLOCK_ASYNC_DIV256          (0x0B << ADC_CCR_PRESC_Pos)

Txc16 anl0_val;
Txc16 anl1_val;
Txc16 anl2_val;


void ADC_Init() {
  __IO uint32_t wait_clk;

	SET_BIT(RCC->AHB2ENR, RCC_AHB2ENR_ADCEN);
 /* PA0 PA1 PA7 Analog Input */
	GPIOA->MODER |= (GPIO_MODER_MODE0 | GPIO_MODER_MODE1 | GPIO_MODER_MODE7);
 /* Disable ADC deep power down mode.  Enable ADC internal voltage regulator. */
	ADC1->CR = ADC_CR_ADVREGEN;
	wait_clk = 532; //ca. 20us  240MHz 9CpuClk
        while (wait_clk != 0) wait_clk--;

 /* 240MHz / 64 = 3.75MHz */
	ADC12_COMMON->CCR = ADC_CLOCK_ASYNC_DIV64;
#if INJECTED
 /* SMP 3.5 ADC clock    16 / 3.75MHz => 4.27us/Channel */
	ADC1->SMPR1 = ADC_SMPR1_SMPPLUS;
	ADC1->SMPR2 = 0;

	ADC1->CFGR = ADC_CFGR_OVRMOD | ADC_CFGR_JAUTO | ADC_CFGR_JQM;
#if 1
 /* 4.27us * 2Channel * 64 Oversamp + 4.27us = 551us */
	ADC1->CFGR2 = ADC_CFGR2_JOVSE | 
			(5 << ADC_CFGR2_OVSR_Pos) |
			(4 << ADC_CFGR2_OVSS_Pos);
#else
 /*  */
	ADC1->CFGR2 = 0;
#endif
	ADC1->OR = 0x01;
 /* 3 channel PA0 PA1 PA7 Analog Input */
	ADC1->JSQR = 2 | (1 << ADC_JSQR_JSQ2_Pos) | (7 << ADC_JSQR_JSQ3_Pos);
	ADC1->CR = ADC_CR_ADVREGEN | ADC_CR_ADEN;

#else  /* end INJECTED */
	ADC1->CFGR = ADC_CFGR_OVRMOD;
	ADC1->CFGR2 = 0;
	ADC1->OR = 0x01;
	ADC1->CR = ADC_CR_ADVREGEN | ADC_CR_ADEN;
#endif

}

int ADC1_ReadAndNextSamp() {
// printf("ADC1->CR=0x%08lx  d=0x%08lx\r\n", ADC1->CR, ADC12_COMMON->CDR);

#if INJECTED
	if (ADC1->CR & ADC_CR_JADSTART) return false;
	anl0_val.u = (uint16_t)ADC1->JDR1;
	anl1_val.u = (uint16_t)ADC1->JDR2;
	anl2_val.u = (uint16_t)ADC1->JDR3;
	ADC1->CR = ADC_CR_ADVREGEN | ADC_CR_ADSTART | ADC_CR_JADSTART | ADC_CR_ADEN;
	return true;
#else
	return false;
#endif
}
