
#include "main.h"
#include "scpi.h"

#define RINGBUF3_SEG_SIZE	((RINGBUF3_MAX_COUNT+1) * 3)
//#define RINGBUF3_B1_ANF		TmcRingBuffer
//#define RINGBUF3_B2_ANF		(TmcRingBuffer + RINGBUF3_SEG_SIZE)


#define RB3BST_SINGLE_BUF	1



__attribute__((aligned(4))) 
uint8_t TmcRingBuffer[RINGBUF3_SEG_SIZE * 2];

static void RingBuf3B_clr(RingBuffer3B *p3, uint32_t items) {
//	p3->stat |= RB3ST_WR_STOP;

	if (items > RINGBUF3S_MAX_COUNT) items = RINGBUF3S_MAX_COUNT;
	p3->items = items + 1;
	p3->stat = 0;
	p3->rstat = 0;
	p3->bstat = 0;
	p3->in_count = 0;
	p3->data1 = TmcRingBuffer;
	p3->data = p3->data1;
	p3->data_anf = p3->data1;
	p3->data1_anf = p3->data1;
	p3->data1_end = p3->data1 + (p3->items * 3);
	p3->data_end = p3->data1_end;

	if (items > RINGBUF3_MAX_COUNT) {
	    p3->data2 = TmcRingBuffer;
	    p3->data2_anf = TmcRingBuffer;
	    p3->data2_end = p3->data1_end;
	    p3->bstat = RB3BST_SINGLE_BUF;
	} else {
	    p3->data2 = TmcRingBuffer + RINGBUF3_SEG_SIZE;
	    p3->data2_anf = TmcRingBuffer + RINGBUF3_SEG_SIZE;
	    p3->data2_end = p3->data2 + (p3->items * 3);
	}
}

uint32_t *RingBuf3B_ClrData_seg(uint32_t *p) {  
  uint32_t i, *pe;

	if (!p) p = (uint32_t*)TmcRingBuffer;
	pe = (uint32_t*)(TmcRingBuffer + (RINGBUF3_SEG_SIZE * 2) - 1);
	i = (uint32_t)pe - (uint32_t)p;

	if (i >= 16384) {
	    for (i=512;i>0;i--) {	/* ~28us */
		*p++ = 0;
		*p++ = 0;
		*p++ = 0;
		*p++ = 0;
		*p++ = 0;
		*p++ = 0;
		*p++ = 0;
		*p++ = 0;
	    }

	} else if (i >= 8192) {
	    for (i=512;i>0;i--) {
		*p++ = 0;
		*p++ = 0;
		*p++ = 0;
		*p++ = 0;
	    }

	} else {
	    if (i >= 4) {
		i >>= 2;
		for (;i>0;i--) *p++ = 0;
	    }
	    p = NULL;
	}
	return p;
}


/*
void RingBuf3B_ClrData(RingBuffer3B *p3) {
  uint32_t i, *p;

	i = RINGBUF3_SEG_SIZE >> 3;
// printf("%ld\r\n", i);

 TEST_PIN_H;
	p = (uint32_t*)TmcRingBuffer;
	while (i) {
	   *p++ = 0;
	   *p++ = 0;
	   *p++ = 0;
	   *p++ = 0;
	   i--;
	}
 TEST_PIN_L;
}
*/
void RingBuf3B_Init(RingBuffer3B *p3, uint32_t items) {

	RingBuf3B_clr(p3, items);
//	RingBuf3B_ClrData(p3);
	p3->trig_pos = 1251;
}

void RingBuf3B_Pos0(RingBuffer3B *p3) {
	p3->data1 = TmcRingBuffer;
	p3->data = p3->data1;
	p3->in_count = 0;
	p3->rstat = 0;
//	p3->rstat &= ~(RB3RST_B2_AKTIV | RB3RST_HAS_DATA | RB3RST_NEW_DATA);
}

void RingBuf3B_TrigOnOff(RingBuffer3B *p3, uint16_t value) {

	RingBuf3B_clr(p3, p3->items - 1);
	p3->stat = value;
	if ((value & RB3ST_USE_TRIGER) == 0) {
	    p3->rstat = RB3RST_HAS_TRIG;
	    TRG_IO_H;
	}
}

uint32_t RingBuf3B_GetCount(RingBuffer3B *p3) {
	if (p3->items == 0) return 0;
	return p3->items - 1;
}

__attribute__((aligned(16))) 
void RingBuf3B_SetTrig(RingBuffer3B *p3) {

	if (p3->rstat & RB3RST_HAS_TRIG) return;
	if (p3->in_count < p3->trig_pos) return;
	p3->in_count = p3->trig_pos;
	if (p3->in_count > p3->items) p3->in_count = p3->items;
	p3->rstat |= RB3RST_HAS_TRIG;
	TRG_IO_H;
}

static void *tmp_p;

__attribute__((aligned(16))) 
void RingBuf3B_Add(RingBuffer3B *p3, uint8_t *val) {
  uint8_t *p;


	if (p3->bstat & RB3BST_SINGLE_BUF) {
	    if ((p3->stat & (RB3ST_RD_AKTIV | RB3ST_FREE_RUN)) == RB3ST_RD_AKTIV) return;
	}

	if (p3->rstat & RB3RST_WR_WAIT) {
	    if ((p3->stat & (RB3ST_RD_AKTIV | RB3ST_SINGLE)) == 0) {
		p3->rstat &= ~RB3RST_WR_WAIT;
		goto LNextBuf;
	    }
	    return;
	}

	p = p3->data;
	*p++ = *val++;
	*p++ = *val++;
	*p++ = *val++;
	if (p >= p3->data_end) p = p3->data_anf;
	p3->data = p;

	p3->in_count++;
	if (p3->in_count < p3->items) return;

	if ((p3->stat & RB3ST_FREE_RUN) || (p3->bstat & RB3BST_SINGLE_BUF)) {
 tmp_p =  p3->data;
	    p3->rstat = RB3RST_NEW_DATA | RB3RST_HAS_DATA;
	    p3->in_count = 0;
	    p3->FrameNum++;
	    return;
	}
	if (p3->stat & RB3ST_USE_TRIGER) {
	    if ((p3->rstat & RB3RST_HAS_TRIG) == 0) return;
	    p3->rstat &= ~RB3RST_HAS_TRIG;
	    TRG_IO_L;
	}

	if (p3->stat & RB3ST_SINGLE) {
	    p3->data1 = p3->data;
//	    p3->Offset = p3->Akt_Offset;
	    p3->rstat = RB3RST_NEW_DATA | RB3RST_HAS_DATA | RB3RST_WR_WAIT;

	} else {
	    if (p3->stat & RB3ST_RD_AKTIV) {
		p3->rstat |= RB3RST_WR_WAIT;
		return;
	    }
LNextBuf:
	    if (p3->rstat & RB3RST_B2_AKTIV) {
		p3->data2 = p3->data;
		p3->data = p3->data1_anf;
		p3->data_anf = p3->data1_anf;
		p3->data_end = p3->data1_end;
//		p3->Offset2 = p3->Akt_Offset;
		p3->rstat = RB3RST_NEW_DATA | RB3RST_HAS_DATA;
	    } else {
		p3->data1 = p3->data;
		p3->data = p3->data2_anf;
		p3->data_anf = p3->data2_anf;
		p3->data_end = p3->data2_end;
//		p3->Offset = p3->Akt_Offset;
		p3->rstat = RB3RST_NEW_DATA | RB3RST_B2_AKTIV | RB3RST_HAS_DATA;
	    }
	}
	p3->in_count = 0;
	p3->FrameNum++;
}



void RingBuf3B_GetDataInit(RingBuffer3B *p3) {

	p3->rstat &= ~RB3RST_NEW_DATA;
	p3->src = NULL;
	p3->stat |= RB3ST_RD_AKTIV;
	if ((p3->stat & RB3ST_FREE_RUN) || (p3->bstat & RB3BST_SINGLE_BUF)) {
	    if (p3->rstat & RB3RST_HAS_DATA) {
		p3->src = p3->data;
	    } else {
		p3->src = p3->data1;
	    }
	    p3->src_anf = TmcRingBuffer;
	    p3->src_end = p3->data1_end;
	    p3->Data_Offset = 0; //p3->Akt_Offset;

	} else if (p3->rstat & RB3RST_HAS_DATA) {
	    if ((p3->rstat & RB3RST_B2_AKTIV) || (p3->stat & RB3ST_SINGLE)) {
		p3->src = p3->data1;
		p3->src_anf = TmcRingBuffer;
		p3->src_end = p3->data1_end;
		p3->Data_Offset = 0; //p3->Offset;
	    } else {
		p3->src = p3->data2;
		p3->src_anf = p3->data2_anf;
		p3->src_end = p3->data2_end;
		p3->Data_Offset = 0; //p3->Offset2;
	    }
	    if (p3->src >= p3->src_end) p3->src = p3->src_anf;

	} else {
	    p3->Data_Offset = 0;
	}
	p3->read_size = p3->items * 3;
}

int32_t RingBuf3B_GetOffset(RingBuffer3B *p3) {
//	if (p3->rstat & RB3RST_HAS_DATA) return p3->Data_Offset;
//	return 0;
	return p3->Data_Offset;
}

int RingBuf3B_GetData(RingBuffer3B *p3, uint8_t *dest, int size) {
  int i;
  uint8_t *src, *endp;

	if ((p3->stat & RB3ST_RD_AKTIV) == 0) return 0;

	if (size > p3->read_size) {
	    size = p3->read_size;
	}

	if (p3->src) {
	    endp = p3->src_end;
	    src = p3->src;

	    if ((src + size) >= endp) {
		for (i=size;i>0;i--) {
		    *dest++ = *src++;
		    if (src >= endp) {
			src = p3->src_anf;
		    }
		}
	    } else {
		if (size == 64) {
		    USB_Move64Byte((uint32_t*)dest, (uint32_t*)src);
		    src += 64;
		} else {
		    for (i=size;i>0;i--) {
			*dest++ = *src++;
		    }
		}
	    }
	    p3->src = src;

	} else {
	    for (i=size;i>0;i--) *dest++ = 0;
	}

	p3->read_size -= size;
	if (p3->read_size == 0) {
	    if ((p3->bstat & RB3BST_SINGLE_BUF) && 
		((p3->stat & (RB3ST_USE_TRIGER | RB3ST_FREE_RUN)) == 0)) 
	    {
		p3->in_count = 0;
		p3->data = TmcRingBuffer;
	    }
	    p3->stat &= ~RB3ST_RD_AKTIV;
	}

	return size;
}
