
#include "main.h"
#include "usb_tmc.h"
#include "usbd_core.h"
#include "usbd_tmc_interface.h"
#include "scpi.h"

uint32_t usbd_cdc_status;
uint32_t usbd_cdc_RxLen;
uint32_t usbd_cdc_RxTimeOut_ms;

uint8_t TmcRxBuffer[TMC_RX_DATA_SIZE];
uint8_t TmcTxBuffer[APP_TX_DATA_SIZE];

void USB_TMC_DefaultInit() {
//  int i;

//	RingBuf3B_Init(&Scpi.p3, RINGBUF3_MAX_COUNT);

//	for (i=0;i<RINGBUF3_MAX_COUNT;i++) {
//	    RingBuf3B_Add(&Scpi.p3, (uint8_t*)&i);
//	}
	RingBuf3B_Init(&rb3b, 63);
}

//uint32_t BuffLength;
//uint32_t UserTxBufPtrIn = 0;/* Increment this pointer or roll it back to
//                               start address when data are received over USART */
//uint32_t UserTxBufPtrOut = 0; /* Increment this pointer or roll it back to
//                                 start address when data are sent over USB */


void USB_TMC_Receive(USBD_HandleTypeDef *pdev, uint32_t Len) {
  uint8_t *data;
  struct usb_tmc_bulk_header *bhin, *bout;
  uint32_t l;

	bhin = (struct usb_tmc_bulk_header *)TmcRxBuffer;
// printf("Receive ID=%d l=%ld Size=%ld\r\n", bhin->MsgID, Len, bhin->spec.msg.transferSize);


	switch (bhin->MsgID) {
	case USB_TMC_MSGID_OUT_DEV_DEP_MSG_OUT:
//		printf("x=%ld\r\n", bhin->spec.msg.transferSize - (Len + tmc_h.RxOffset - sizeof(struct usb_tmc_bulk_header)));

		if ((Len == 64) && 
		    (bhin->spec.msg.transferSize > (Len + tmc_h.RxOffset - sizeof(struct usb_tmc_bulk_header)))) {
		    if ((tmc_h.RxOffset + 64) > TMC_RX_DATA_SIZE) {
			tmc_h.RxState =  TMC_RXST_ERROR;
		    } else {
		        tmc_h.RxOffset += 64;
		    }
		    USBD_LL_PrepareReceive(&USB_Dev, CDC_OUT_EP, TmcRxBuffer + tmc_h.RxOffset,
					CDC_DATA_FS_OUT_PACKET_SIZE);
		    return;
		}

#if 0
		if (bhin->bTag != ~(bhin->bTagInverse)) {
			printf("ignoring invalid: bTag != bTagInverse?!!");
			return;
		}
#endif

		if ((tmc_h.RxState & TMC_RXST_ERROR) == 0) {
		    data = TmcRxBuffer + sizeof(struct usb_tmc_bulk_header);
		    data[bhin->spec.msg.transferSize] = 0;
		    
//		    Scpi.stat = 0;
//		    Scpi.tx_data = TmcTxBuffer + sizeof(struct usb_tmc_bulk_header);
//		    Scpi.tx_data_len = 0;
// TEST_PIN2_ON;
		    scpi_line_set(data, bhin->spec.msg.transferSize, &Scpi);
// TEST_PIN2_OFF;
		}

		tmc_h.RxState &= ~TMC_RXST_ERROR;
		tmc_h.RxOffset = 0;
	        USB_TMC_ReceivePacket(&USB_Dev);
		return;

	case USB_TMC_MSGID_OUT_REQUEST_DEV_DEP_MSG_IN:
		tmc_h.RxOffset = 0;
//		printf("Size=%ld Atr=0x%02x\r\n", bhin->command_specific.dev_dep_msg_out.transferSize, bhin->command_specific.dev_dep_msg_out.bmTransferAttributes);

		bout = (struct usb_tmc_bulk_header *)TmcTxBuffer;
		bout->MsgID = USB_TMC_MSGID_OUT_REQUEST_DEV_DEP_MSG_IN;
		bout->bTag = bhin->bTag;
		bout->bTagInverse = ~bout->bTag;
		bout->reserved = 0;
		bout->spec.req_msg.TermChar = 0;

		l = Scpi.tx_data - (TmcTxBuffer + sizeof(struct usb_tmc_bulk_header));
// printf("Txd len=%ld\r\n", l);
		if (l > 0) {
		    bout->spec.req_msg.transferSize = l + 1;
		    bout->spec.req_msg.bmTransferAttributes = 0;
		} else {
		    bout->spec.req_msg.transferSize = 0;
		    bout->spec.req_msg.bmTransferAttributes = USB_TMC_STATUS_SUCCESS;
//		    data = TmcTxBuffer + sizeof(struct usb_tmc_bulk_header);
//		    data[0] = 'E';
//		    data[1] = 'R';
//		    data[2] = 'R';
//		    data[3] = 0;
		}

		l = ((bout->spec.req_msg.transferSize + 3) >> 2) << 2;
		CDC_Transmit(TmcTxBuffer, sizeof(struct usb_tmc_bulk_header) + l);
		Scpi.tx_data = TmcTxBuffer + sizeof(struct usb_tmc_bulk_header);


		USB_TMC_ReceivePacket(&USB_Dev);
		return;
#if 0
	case USB_TMC_MSGID_OUT_VENDOR_SPECIFIC_OUT:
		printf("vendor_out (UNHANDLED)\r\n");
		return;
	case USB_TMC_MSGID_OUT_REQUEST_VENDOR_SPECIFIC_IN:
		printf("req_vendor_in (UNHANDLED)\r\n");
		return;
#endif
	}
#if DEBUG_CONSOLE_ON
	printf("bhin->MsgID=%d (UNHANDLED)\r\n", bhin->MsgID);
#endif
}



int8_t CDC_Transmit(uint8_t* Buf, uint32_t Len) {
//  USB_TMC_HandleTypeDef *tmc_h;


// printf("Transmit Len=%d\r\n", (int)Len);

	USB_TMC_SetTxBuffer(&USB_Dev, (uint8_t*)Buf, Len);

	if(USB_TMC_TransmitPacket(&USB_Dev) == USBD_OK) {
	} else {
//	    printf("ERROR: USB_TMC_TransmitPacket\r\n");
	}

//	tmc_h = (USB_TMC_HandleTypeDef *)USB_Dev.pClassDataCmsit;
//	if (tmc_h) 
	tmc_h.RxOffset = 0;
	    USBD_LL_PrepareReceive(&USB_Dev, CDC_OUT_EP, TmcRxBuffer,
                                 CDC_DATA_FS_OUT_PACKET_SIZE);

	
//	USB_TMC_SetRxBuffer(&USB_Dev, TmcRxBuffer);

	return 0;
}

void TMC_TransmitNext(PCD_HandleTypeDef *hpcd, PCD_EPTypeDef *ep) {
// printf("c=%ld %ld 0x%06x\r\n", ep->xfer_count, ep->xfer_len, (int)ep->xfer_buff);
// int sidx;
	if (ep->num != 2) return;
	if (ep->xfer_count >= 128) {
// TEST_PIN_H;
	    RingBuf3B_GetData(&rb3b, TmcTxBuffer, 64);
	    ep->xfer_buff = TmcTxBuffer;
// TEST_PIN_L;
	}
}
