#include <stddef.h>
#include <stdbool.h>
#include <stdint.h>

#include "main.h"
#include "utils.h"

int MoveStr(char *dest, const char *src) {
  const char *s;
  char *d;

	if (!dest || !src) return 0;
	s = src;
	d = dest;
//	while (*s) *d++ = *s++;
	while ((*d++ = *s++));
	*d = 0;
	return s - src;
}

int Uint8ToStr(char *dest, uint8_t val) {
  char *pc;
  uint8_t digit;

	pc = dest;
	*pc = 0;
	if (val >= 200) {
	    *pc = '2'; pc++;
	    val -= 200;
	} else if (val >= 100) {
	    *pc = '1'; pc++;
	    val -= 100;
	}

	digit = val / 10;
	if (digit || *dest) {
	    *pc = digit | 0x30; pc++;
	    val -= digit * 10;
	}
	*pc = val | 0x30; pc++;
	*pc = 0;
	return pc - dest;
}

int Int8ToStr(char *dest, int8_t val) {
  char *pc;

	pc = dest;
	if (val & 0x80) {
	    *pc = '-'; 
	    pc++;
	    val = -val;
	}
	pc += Uint8ToStr(pc, val);
	return pc - dest;
}

int Uint16ToStr(char *dest, uint16_t val) {
  char *pc;
  uint8_t digit;

	pc = dest;
	*pc = 0;
	digit = val / 10000;
	if (digit) {
	    *pc = digit | 0x30; pc++;
	    val -= digit * 10000;
	}
	digit = val / 1000;
	if (digit || *dest) {
	    *pc = digit | 0x30; pc++;
	    val -= digit * 1000;
	}
	digit = val / 100;
	if (digit || *dest) {
	    *pc = digit | 0x30; pc++;
	    val -= digit * 100;
	}

	digit = val / 10;
	if (digit || *dest) {
	    *pc = digit | 0x30; pc++;
	    val -= digit * 10;
	}
	*pc = val | 0x30; pc++;
	*pc = 0;
	return pc - dest;
}

int Int16ToStr(char *dest, int16_t val) {
  char *pc;

	pc = dest;
	if (val & 0x8000) {
	    *pc = '-'; 
	    pc++;
	    val = -val;
	}
	pc += Uint16ToStr(pc, val);
	return pc - dest;
}

int Uint32ToStr(char *dest, uint32_t val) {
  uint32_t k;
  uint8_t digit;
  char *pc;

	k = 1000000000;
	while (k > val) k /= 10;

	pc = dest;
	*pc = 0;
	while (k > 6) {
	    digit = val / k;
	    *pc = digit | 0x30; pc++;
	    val -= digit * k;
	    k /= 10;
	}
	*pc = val | 0x30; pc++;
	*pc = 0;
	return pc - dest;
}  

#define bNUM	1
#define bHEX	(1<<1)
#define bNEG	(1<<2)

int ppctoint(const char **ppc) {
  int res, Mk;
  const char *pc;
	if (!ppc || !*ppc) return 0;
	pc = *ppc;
	Mk = res = 0;

	while (*pc) {
	    if (!Mk && (*pc == '0') && ((pc[1] == 'x') || (pc[1] == 'X'))) {
		Mk |= bHEX;
		pc++;
	    } else if ((*pc >= '0') && (*pc <= '9')) {
		Mk |= bNUM;
		if (Mk & bHEX) res <<= 4; else res *= 10;
		res += *pc & 0x0f;

	    } else if ((Mk & bHEX) && (((*pc >= 'A') && (*pc <= 'F')) || 
				       ((*pc >= 'a') && (*pc <= 'f')))) {
		res = (res << 4) + ((*pc - 7) & 0x0f);

	    } else if (*pc == '-') {
		if (Mk) break;
		Mk = bNEG;
		if ((pc[1] == '0') && (pc[2] == 'x')) {
		    Mk |= bHEX;
		    pc += 2;
		}
	    } else if ((*pc == ' ') || (*pc == 0x09)) {
		if (Mk) break; //Leerzeichen am Anfang erlaubt
	    } else if (!Mk && (*pc == '$')) {
		Mk = bHEX;
	    } else {
		break;
	    }
	    pc++;
	}
	if (Mk & bNEG) res = -res;
	*ppc = pc;
	return res;
}

int pctoint(const char *pc) {
  const char *cpc = pc;
	return ppctoint(&cpc);
}

#define MF_COMMA	1
#define MF_POS		(1<<1)
#define MF_NEG		(1<<2)
#define MF_NUM		(1<<3)
#define MF_EXP		(1<<4)
#define MF_EXPPOS	(1<<5)
#define MF_EXPNEG	(1<<6)
#define MF_EXPNUM	(1<<7)
#define MF_EXP_ALL	(MF_EXP|MF_EXPPOS|MF_EXPNEG|MF_EXPNUM)

float ppctofloat(const char **ppc) {
  int Mk, exp;
  const char *pc, *pc_exp;
  float res, dez;

	if (!ppc || !*ppc) return 0;
	pc = *ppc;
	pc_exp = NULL;
	Mk = 0;
	dez = 0.1;
	res = 0;
	exp = 0;
	while (*pc) {
	    if ((*pc >= '0') && (*pc <= '9')) {
		if ((Mk & MF_EXP) == 0) {
		    Mk |= MF_NUM;
		    if (Mk & MF_COMMA) {
			res += (*pc & 0x0f) * dez;
			dez *= 0.1;
		    } else {
			res = (res * 10.0) + (*pc & 0x0f);
		    }

		} else {
		    if (!(Mk & (MF_EXPPOS|MF_EXPNEG))) break;
		    Mk |= MF_EXPNUM;
		    exp = (exp * 10) + (*pc & 0x0f);
		}

	    } else if (*pc == '-') {
		if (Mk & MF_EXP) {
		    if ((Mk & MF_EXP_ALL) != MF_EXP) break;
		    Mk |= MF_EXPNEG;
		} else {
		    if (Mk & (MF_COMMA|MF_POS|MF_NEG|MF_NUM)) break;
		    Mk |= MF_NEG;
		}

	    } else if (*pc == '+') {
		if (Mk & MF_EXP) {
		    if ((Mk & MF_EXP_ALL) != MF_EXP) break;
		    Mk |= MF_EXPPOS;
		} else {
		    if (Mk & (MF_COMMA|MF_POS|MF_NEG|MF_NUM)) break;
		    Mk |= MF_POS;
		}

#if 1
	    } else if (*pc == '.') {
#else
	    } else if ((*pc == '.') || (*pc == ',')) {
#endif
		if (Mk & (MF_COMMA|MF_EXP)) break;
		Mk |= MF_COMMA;

	    } else if (((*pc == 'E')||(*pc == 'e')) && 
			((Mk & (MF_NUM|MF_EXP)) == MF_NUM)) {
		pc_exp = pc;
		Mk |= MF_EXP;

	    } else if ((*pc == ' ') || (*pc == 0x09)) {
		if (Mk) break; //Leerzeichen am Anfang erlaubt

	    } else {
		break;
	    }
	    pc++;
	}

	if (Mk & MF_EXP) {
	    if (Mk & MF_EXPNUM) {
		if (Mk & MF_EXPNEG) {
		    if (exp > 38+15) exp = 38+15;
		    while (exp >= 10)  {res *= 1E-10;  exp -= 10;};
		    while (exp >= 3)   {res *= 0.001;  exp -= 3;};
		    while (exp >= 1)   {res *= 0.1;    exp -= 1;};
		} else {
		    if (exp > 37+15) exp = 37+15;
		    while (exp >= 10)  {res *= 1E+10;  exp -= 10;};
		    while (exp >= 3)   {res *= 1000.0; exp -= 3;};
		    while (exp >= 1)   {res *= 10.0;   exp -= 1;};
		}
	    } else {
		pc = pc_exp;
	    }
	}
	if (Mk & MF_NEG) res = -res;
	*ppc = pc;
	return res;
}

float pctofloat(const char *pc) {
  const char *ppc = pc;
	return ppctofloat(&ppc);
}

typedef union {
  float  f;
  uint32_t u;
  uint16_t u16[2];
} Txxxf32;

int TestFloatHasError(float *f) {
 Txxxf32 xc32;
 uint16_t e;

	xc32.f = *f;
	e = (xc32.u16[1] >> 7) & 0xff;
	if ((e == 0xff) || (!e && xc32.u)) return 1;
	return 0;
}

//Achtung *ret min 16 char's
void floattoppc(char **ppc, float w, int decipoint) {
  float d10;
  int m, nk, e;
  char *pc;

	if (!ppc || !*ppc) return;
	pc = *ppc;
	e = 0;

	nk = decipoint & 0x000f;
	if (nk > 9) nk = 9;
	if (TestFloatHasError(&w)) {
	    *pc++ = 'n';
	    *pc++ = 'a';
	    *pc++ = 'n';
	    *pc++ = ' ';

	} else if (w == 0.0) {
	    *pc++ = '0';
	    if (nk > 0) {
		*pc++ = '.';
		while (nk) {*pc++ = '0'; nk--;}
	    }

	} else {
	    if (w < 0.0) {*pc = '-'; pc++; w = -w;}

	    if (decipoint & DECIPOINT_EXP) {
//		if (!nk) nk = 7;
		if (w >= 10.0) {
		    while (w >= 1E+10) {w *= 1E-10; e += 10;}
		    while (w >= 1E+3) {w *= 1E-3; e += 3;}
		    while (w > 9.999999) {w *= 0.1; e += 1;}
		} else if (w < 1.0) {
		    while (w < 1E-10) {w *= 1E+10; e -= 10;}
		    while (w < 1E-3) {w *= 1E+3; e -= 3;}
		    while (w < 0.9999999) {w *= 10; e -= 1;}
		}
	    }

	    d10 = 0.5;
	    m = nk;
	    while (m--) d10 /= 10.0;
	    w += d10;
	    if (w >= 1) {
		while (w > 1E+10) {e++; w /= 10.0;}
		d10 = 100.0;
		while (d10 < w) d10 *= 100.0;
		while ((d10 >= 1) && (d10 > w)) d10 /= 10.0;
		while (d10 >= 1) {
		    m = w / d10;
		    *pc++ = (m + 0x30);
		    w = w - (d10 * m);
		    d10 /= 10.0;
		}
	    } else {
		*pc++ = '0';
	    }

	    if (nk > 0) {
		*pc++ = '.';
		while (nk--) {
		    w *= 10.0;
		    m = w;
		    *pc++ = (m + 0x30);
		    w = w - m;
		}
	    }

	    if (e) {
		*pc++ = 'E';
		if (e > 0) *pc++ = '+';
		pc += Int8ToStr(pc, e);
	    }
	}
	*pc = 0;
	*ppc = pc;
}

int floattopc(char *pc, float w, int decipoint) {
  char *pcn = pc;
	floattoppc(&pcn, w, decipoint);
	return pcn-pc;
}

void chkf(float *pf, float min, float max, float def) {
#ifndef SIM_PC
  float tf;

	if (TestFloatHasError(pf)) goto Lerr;
	__ASM ("ldr	%0, [%1, #0]" : __CMSIS_GCC_OUT_REG (tf) : __CMSIS_GCC_USE_REG (pf) );
	if ((tf >= min) && (tf <= max)) return;
Lerr:
	__ASM ("str	%1, [%0, #0]" : : __CMSIS_GCC_USE_REG (pf), __CMSIS_GCC_USE_REG (def) );
#else
/* HardFault zB.: Value = 0xFFFFFFFF !!!!!!! ????? !!!!!!! gcc-12.4.0 */
	if (TestFloatHasError(pf) || (*pf < min) || (*pf > max)) *pf = def;
#endif
}
