
#include "main.h"
#include <stdio.h>
#include "libh523.h"
#include "usbd_core.h"
#include "tmc.h"
#include "usb_tmc.h"
#include "scpi.h"
#include "oszi.h"
#include "utils.h"

__attribute__((aligned(4))) 
USBD_HandleTypeDef USB_Dev;
__attribute__((aligned(4))) 
PCD_HandleTypeDef hpcd;

__attribute__((aligned(4))) 
uint8_t FlashBuffer[sizeof(TFlashData) + 16];

uint16_t ExecCmdBef;

RingBuffer3B rb3b;

__attribute__((aligned(4))) 
float InDC, UsbV, CmdTestFloat, CmdTestFloatResult, InDC_DigitV;
uint32_t CmdTestBef;
uint32_t LoopCount;
int32_t InDC_count;
int VOPst;
TIRR16b_32 IRR_InPwr;

static void* GetFlashDataAdr(uint16_t *fd_seg) {
  int i, r_seg;
  uint32_t *pu32;
  uint32_t *result;
  Txc32 xc32;

	pu32 = (uint32_t*)FLASH_DATA_ADR;
	result = NULL;

	r_seg = 0;
	i = 0;
	while (i<FLASH_MAX_USER_SEC) {
	    xc32.u = *pu32;
	    if (xc32.u16[1] != FLASH_DATA_VER) break;
	    if (xc32.u16[0] == FLASH_DATA_KENN(i)) {
		r_seg = i;
		result = pu32;
	    }
	    i++;
	    pu32 += (FLASH_MAX_USER_SEC_SIZE / 4);
	}
	if (fd_seg) *fd_seg = r_seg;
	return result;
}

static uint8_t SaveFlashData() {
  uint32_t *FlashAddress, *DataAddress;
  uint16_t fd_seg, QuadCount;
  TFlashData *fd;
  uint8_t erase;

	erase = false;
	fd_seg = 0;
	FlashAddress = GetFlashDataAdr(&fd_seg);

	if (FlashAddress) {
	    if (fd_seg >= (FLASH_MAX_USER_SEC - 1)) {
		erase = true;
	    } else {
		fd_seg++;
		FlashAddress += (FLASH_MAX_USER_SEC_SIZE / 4);
		if (*FlashAddress != 0xFFFFFFFF) erase = true;
	    }
	} else {
	    FlashAddress = (uint32_t*)FLASH_DATA_ADR;
	    if (*FlashAddress != 0xFFFFFFFF) erase = true;
	}

	FlashClrError();

// printf("SAVE: Adr=0x%08lx erase=%d seg=%d\r\n", (uint32_t)FlashAddress, erase, fd_seg);
	if (erase) {
	    if (!FlashErase(FLASH_DATA_SECTOR)) return false;
	    FlashAddress = NULL;
	    fd_seg = 0;
	}

	fd = (void*)FlashBuffer;
	fd->kenn = FLASH_DATA_KENN(fd_seg);
	fd->ver = FLASH_DATA_VER;

	if (!FlashAddress) FlashAddress = (uint32_t*)FLASH_DATA_ADR;
	DataAddress = (void*)FlashBuffer;

// printf("SAVE0: Adr=0x%08lx -> 0x%08lx\r\n", (uint32_t)DataAddress, (uint32_t)FlashAddress);

	if (FlashUnlock() && FlashWait(10)) {
	    WRITE_REG(ICACHE->FCR, ICACHE_FCR_CBSYENDF);
	    CLEAR_BIT(ICACHE->CR, ICACHE_CR_EN);

	    QuadCount = (sizeof(TFlashData) + 15) / 16;
// printf("QuadCount=%d\r\n", QuadCount);
	    for (;QuadCount>0;QuadCount--) {
		FlashProgramQuadWord(&FlashAddress, &DataAddress);
// printf("SAVE: Adr=0x%08lx -> 0x%08lx\r\n", (uint32_t)DataAddress, (uint32_t)FlashAddress);
		FlashWait(1000);
		CLEAR_BIT(FLASH_NS->NSCR, FLASH_CR_PG);
	    }

	    FlashLock();
	    SET_BIT(ICACHE->CR, ICACHE_CR_EN);
	}
	return true;
}

static void LoadFlashData() {
  uint32_t *FlashAddress, *DataAddress;
  uint16_t u32Count;

	FlashAddress = GetFlashDataAdr(NULL);
	if (!FlashAddress) return;

	DataAddress = (void*)FlashBuffer;
	u32Count = (sizeof(TFlashData) + 3) / 4;

	for (;u32Count>0;u32Count--) {
	    *DataAddress++ = *FlashAddress++;
	}
}


int main(void) {
//  char buf[32];
  TFlashData *fd;

	fd = (void*)FlashBuffer;

	CONSOLE_Init();
#if DEBUG_CONSOLE_ON
	printf("hello world.\r\n");
#endif

	VOPst = 3;

//	PIN_5VEN_ON;
//	HAL_Delay(20);	// Wait 20ms

//	PIN_OPVEN_ON;
//	HAL_Delay(15);

	ADC_Init();
	DAC_Init();
	Ads127l01_Init();

	LoadFlashData();

// RCC->PLL1CFGR &= ~RCC_PLL1CFGR_PLL1FRACEN;
// RCC->PLL1FRACR = 0xf000;
// RCC->PLL1CFGR |= RCC_PLL1CFGR_PLL1FRACEN;

	chkf(&fd->TempOffs, -30.0, 30.0, 0.0);
	chkf(&fd->AdcVpC, -0.001, 0.001, 0.0);

	if ((fd->InDC_DigitV < 100) || (fd->InDC_DigitV == 0xFFFF)) {
	    fd->InDC_DigitV = INDC_DIGIT_V * 1000000.0;
	}
	InDC_DigitV = (float)fd->InDC_DigitV * 0.000001;

  /* Init USB */
#if 1
  /* Init Device Library */
	USBD_LL_Init(&USB_Dev, NULL);
	USB_TmcDescInit(&USB_Dev);
  /* Add Supported Class */
	USB_Dev.pClass = &USBD_CDC;
	USB_Dev.NumClasses = 1;

//	USB_TMC_DefaultInit();
	Oszi_ConfigInit();
  /* Start Device Process */
	USBD_LL_Start(&USB_Dev);
	NVIC_SetPriority(USB_DRD_FS_IRQn, 6);
#endif


	TMP236SetTempOffset(fd->TempOffs);

	HAL_Delay(1);
	ADC1_ReadAndNextSamp();
	HAL_Delay(10);

#if DEBUG_CONSOLE_ON
	printf("kenn=0x%04x ver=%d\r\n", fd->kenn, fd->ver);
#endif
/*
	floattopc(buf, fd->V500mV_Mul, 6);
 printf("500m=%s ", buf);
*/

// LED_RED_OFF;
 LED_GREEN_ON;

 Oszi_ExtTrigOutEn(false);

  uint32_t tmp_loop, *p_rb3; //, DAC_Val;
  int loop_100ms, ui, ms_mk;
  int Step10ms, prmk;
  float tf;
  int32_t AnlVal, i32, TempIRR_X, TempIRR_Y, AnyOffset;
//  TIIR16b_32 K2_IRR, K3_IRR, K4_IRR;

	p_rb3 = NULL;
	TempIRR_X = 0;
	TempIRR_Y = 0;
	AnyOffset = 0;
	ms_mk = 0;
	tmp_loop = 0;
	loop_100ms = 100;
	Step10ms = -1;
	prmk = 1;

	RelaisTvKorrInit();

// A=0.001953125 B=0.996093750
	irr16b_32_k3FFF_init(&IRR_InPwr, 64);


	while (1) {
//	    __asm volatile ("nop;");
#if 0
	    if (CONSOLE_ReceiverIsReady()) {
		buf[0] = CONSOLE_ReadByte();
		if (buf[0]) {
		    CONSOLE_Write(buf, 1);
		}
	    }
#endif


	    tmp_loop++;
	    ui =  TimerCount_ms;
	    if (ui != ms_mk) {
		ms_mk = ui;
		loop_100ms--;
		if (loop_100ms <= 0) {
		    LoopCount = tmp_loop;
		    tmp_loop = 0;
		    loop_100ms = 100;

		}

		Step10ms++;
		switch (Step10ms) {
		case 0:
		    i32 = TempIRR_Y >> 8;
		    tf = TMP236GetTemp(i32);
/*
 if (prmk) {
  char buf[32];
  floattopc(buf, tf, 2);
  printf("%sC 0x%04lx 0x%04lx 0x%04x\r\n", buf, i32, TempIRR_Y, anl0_val.u);
  prmk = 0;
 }
*/
		    tf -= 25.0;
		    tf *= fd->AdcVpC;
		    AnyOffset = tf * ADS127L01_1p_DIGV;
		    Oszi_KorrTempAdcOffset();
		    break;

		case 1:
		    if (VOPst) {
		        if (UsbV > 4.5) {
			    PIN_5VEN_ON;
			    if (VOPst == 1) {
				PIN_OPVEN_ON;
				LED_RED_OFF;
			    }
			    VOPst--;
			}
		    }
		    break;
		case 2:
		    AnyOffset += RelaisTvKorr();
		    break;
		case 3:
		    break;
		case 4:
		    break;
		case 5:
		    OsziConfig.TempOffset = AnyOffset;
		    break;
		case 6:
		    break;
		case 7:
		    break;
		case 8:
		    break;
		case 9:
		default:
		    if (!VOPst) {
			if (Scpi.err[0]) {
			    LED_RED_ON;
			} else {
			    LED_RED_OFF;
			}
		    }
		    Step10ms = -1;
		}

		ADC1_ReadAndNextSamp();

/* 3.25V / 16384 * (2k2 / 2k2 + 1) */
		UsbV = (float)(anl1_val.i) * 0.000396728;



  int32_t res;

		i32 = anl0_val.u * 4;
		res = i32 + TempIRR_X;
		TempIRR_X = i32;
		i32 = (TempIRR_Y * 31) >> 5;
		TempIRR_Y = res + i32;

		InDC = (float)(anl2_val.i + fd->InDC_Offset - 8192) * InDC_DigitV;
#if 1
		if ((INP_ATT_STAT == 0) && (INP_DC_STAT != 0)) {
#ifdef INIMP_2K5
		    tf = InDC * InDC * 1000.0 / 250.0;
#elif defined(INIMP_10K)
		    tf = InDC * InDC * 1000.0 / 1000.0;
#else
		    tf = InDC * InDC * 1000.0 / 50.0;
#endif
		    i32 = tf;
		    i32 = irr16b_32_k3FFF(&IRR_InPwr, i32);

#ifdef INIMP_2K5
		    if ((fabsf(InDC) > 20.0) || (i32 >= 100)) {
			Oszi_SetV_DivStr("5");
			AddError(ERR_OVERVOLTAGE);
		    }
#else
		    if ((fabsf(InDC) > 12.0) || (i32 >= 750)) {
			Oszi_SetInCoup("AC");
			AddError(ERR_OVERVOLTAGE);

		    } else if (i32 >= 550) {
			Oszi_SetV_DivStr("1");
			AddError(ERR_OVERVOLTAGE);
		    }
#endif

/*
 if (prmk) {
  char buf[32];
  floattopc(buf, InDC, 2);
  printf("%sV 0x%04lx 0x%04lx\r\n", buf, i32, AnlVal);
  prmk = 0;
 }
*/
		} else {
		    irr16b_32_k3FFF(&IRR_InPwr, 0);
		}
#endif

	    }

	    if (ExecCmdBef) {
		if (ExecCmdBef & EXEC_SAVE_CAL_DATA) {
		    SaveFlashData();
		    ExecCmdBef &= ~EXEC_SAVE_CAL_DATA;

		} else if (ExecCmdBef & EXEC_CLR_BUF_DATA) {
// TEST_PIN_H;
		    p_rb3 = RingBuf3B_ClrData_seg(p_rb3);
// TEST_PIN_L;
		    if (!p_rb3) {
			Ads127l01_Start();
			ExecCmdBef &= ~EXEC_CLR_BUF_DATA;
		    }
		}
	    }

	    if (CmdTestBef == 1) {
		prmk++;

	    } else if (CmdTestBef == 2) {

	    }
	    if (CmdTestBef) CmdTestBef = 0;
#if 1
	    if (Ads127l01_GetAnlVal(&AnlVal)) {
		Oszi_DmmV_Conv(AnlVal);
	    }
#endif
	}
}

#if 1
void HardFault_Handler(void) {
  uint32_t sp; //, fpscr;
//  uint32_t fpscr;
  __ASM volatile ("MRS %0, msp" : "=r" (sp) );
//  __ASM volatile ("MRS %0, psp" : "=r" (psp) );
//  __ASM volatile ("VMRS %0, fpscr" : "=r" (fpscr) );

//  uint32_t *pfpscr = (uint32_t *)(sp + 0x60);

/*
	printf("HFH FPSCR=0x%08lx SP=0x%08lx\r\n", *pfpscr, sp);

	fpscr = *pfpscr;
	if (fpscr & 0x8F) {
	    printf("ERROR: Div 0\r\n");
	    *pfpscr = fpscr & ~(0x0000008F);
	    return;
	}
*/
#if DEBUG_CONSOLE_ON
	printf("SCB->HFSR=0x%08lx SP=0x%08lx\r\n", SCB->HFSR, sp);
#endif
	LED_RED_ON;
	while (1);
}
#endif


#if 1
void FPU_IRQHandler(void) {
  uint32_t *pfpscr;

	LED_RED_ON;
//	while (1);

//  uint32_t sp;
//  __ASM volatile ("MRS %0, msp" : "=r" (sp) );

	__get_FPSCR();
	pfpscr = (uint32_t *)(FPU->FPCAR+0x40);
#if DEBUG_CONSOLE_ON
	printf("FH FPSCR=0x%08lx\r\n", *pfpscr);
#endif
	*pfpscr = *pfpscr & (uint32_t)~0x8F;;

	LED_RED_OFF;
}
#endif
