
#include "main.h"
#include "usbd_tmc_interface.h"
#include "scpi.h"
#include "utils.h"
#include "oszi.h"
#include "libh523.h"

TOsziConfig OsziConfig;
static float DmmValue;

#define CP_UNALIGN 1

#define END(X) (!pc[X] || (pc[X] == ' '))

#define CP1(a) (pc[0] == a)
#if CP_UNALIGN
#define CP2(a,b) (*(int16_t*)pc == (int16_t)(a|(b << 8)))
//#define CP2(a,b) ((pc[0] == a)&&(pc[1] == b))
#define CP3(a,b,c) (CP2(a,b)&&(pc[2] == c))
#define CP4(a,b,c,d) (*(int32_t*)pc == (int32_t)(a|(b << 8)|(c << 16)|(d << 24)))
#define CP5(a,b,c,d,e) (CP4(a,b,c,d)&&(pc[4] == e))
#else
#define CP2(a,b) ((pc[0] == a)&&(pc[1] == b))
#define CP3(a,b,c) ((pc[0] == a)&&(pc[1] == b)&&(pc[2] == c))
#define CP4(a,b,c,d) ((pc[0] == a)&&(pc[1] == b)&&(pc[2] == c)&&(pc[3] == d))
#define CP5(a,b,c,d,e) ((pc[0] == a)&&(pc[1] == b)&&(pc[2] == c)&&(pc[3] == d)&&(pc[4] == e))
#endif

#ifdef INIMP_10K
#define VDIV_COUNT	7
#else
#define VDIV_COUNT	6
#endif

typedef struct {
  const float    V_Div;
  const int16_t  *pOffs;
  const float    *pVmul;
  const uint16_t Kio;
  const uint16_t DivCount;	/* 1V/Div * 8 = +-4V */
} TOsziVpDiv;

#define CALD ((TFlashData*)(void*)FlashBuffer)

__attribute__((aligned(4))) 
const TOsziVpDiv Tab_OsziVpDiv[VDIV_COUNT] = {
#ifdef INIMP_10K
  {5.0,  &CALD->V5000mV_Offs, &CALD->V5000mV_Mul, 0x01, 10},
  {2.0,  &CALD->V2000mV_Offs, &CALD->V2000mV_Mul, 0x03, 10},
  {1.0,  &CALD->V1000mV_Offs, &CALD->V1000mV_Mul, 0x05, 10},
  {0.5,  &CALD->V500mV_Offs,  &CALD->V500mV_Mul,  0x00, 10},
  {0.2,  &CALD->V200mV_Offs,  &CALD->V200mV_Mul,  0x02, 10},
  {0.1,  &CALD->V100mV_Offs,  &CALD->V100mV_Mul,  0x04, 10},
  {0.05, &CALD->V50mV_Offs,   &CALD->V50mV_Mul,   0x08, 10}
#else
  {1.0,  &CALD->V1000mV_Offs, &CALD->V1000mV_Mul, 0x01, 10},
  {0.5,  &CALD->V500mV_Offs,  &CALD->V500mV_Mul,  0x00, 10},
  {0.2,  &CALD->V200mV_Offs,  &CALD->V200mV_Mul,  0x02, 10},
  {0.1,  &CALD->V100mV_Offs,  &CALD->V100mV_Mul,  0x04, 10},
  {0.05, &CALD->V50mV_Offs,   &CALD->V50mV_Mul,   0x08, 10},
  {0.02, &CALD->V20mV_Offs,   &CALD->V20mV_Mul,   0x0E, 16}
#endif
};

/* 0x007ff000 = 8384512.0 */
#define TRIG_MAX_POS	8384512
#define TRIG_MAX_POS_F	8384512.0
/* 0xFF801000 = -8384512.0 */
#define TRIG_MAX_NEG	-8384512
#define TRIG_MAX_NEG_F	-8384512.0

static void set_trig_source() {
  uint16_t stat;
  uint16_t TrigMod;

	stat = 0;
//	if (OsziConfig.TrigFkt & OSZI_TRIGFKT_ON) stat = RB3ST_USE_TRIGER; else
//	if (OsziConfig.TrigFkt & OSZI_TRIGFKT_FREERUN) stat = RB3ST_FREE_RUN;
//	if (OsziConfig.TrigFkt & OSZI_TRIGFKT_SINGLE)  stat |= RB3ST_SINGLE;

	switch (OsziConfig.TrigFkt & OSZI_TRIGFKT_MASK) {
	case OSZI_TRIGFKT_ON:
	    stat = RB3ST_USE_TRIGER;
	    break;
	case OSZI_TRIGFKT_SINGLE:
	    stat = RB3ST_SINGLE;
	    break;
	case OSZI_TRIGFKT_ON_SINGLE:
	    stat = RB3ST_USE_TRIGER | RB3ST_SINGLE;
	    break;
	case OSZI_TRIGFKT_FREERUN:
	    stat = RB3ST_FREE_RUN;
	    break;
	case (OSZI_TRIGFKT_EXT | OSZI_TRIGFKT_ON):
	    stat = RB3ST_USE_TRIGER;
	    break;
	}

	RingBuf3B_TrigOnOff(&rb3b, stat);
	if (stat & RB3ST_USE_TRIGER) {
	    if (OsziConfig.TrigFkt & OSZI_TRIGFKT_NEG) {
		TrigMod = TRIG_HL;
	    } else {
		TrigMod = TRIG_LH;
	    }
	    if (OsziConfig.TrigFkt & OSZI_TRIGFKT_EXT) {
		TrigMod |= TRIG_EXT;
	    }

	} else {
	    TrigMod = 0;
	}
	Ads127l01_SetTrigger(OsziConfig.TrigHV, OsziConfig.TrigLV, TrigMod);
}

static void set_adc_max_V() {
  float tf;
  int32_t i32;

	OsziConfig.dac_V = OsziConfig.V_Div * (float)OsziConfig.V_DivCount;
	OsziConfig.dac_digit_V = OsziConfig.dac_V * 
				 OsziConfig.dac_Vmul * ADC_BASE_KORR;

	tf = OsziConfig.TrigV / OsziConfig.dac_digit_V;
	if (tf > TRIG_MAX_POS_F) tf = TRIG_MAX_POS_F;
	if (tf < TRIG_MAX_NEG_F) tf = TRIG_MAX_NEG_F;
	i32 = tf;
	i32 += OsziConfig.GainOffset;
	i32 += OsziConfig.TempOffset;

	OsziConfig.TrigHV = i32 + OsziConfig.TrigToll;
	if (OsziConfig.TrigHV >  TRIG_MAX_POS) OsziConfig.TrigHV =  TRIG_MAX_POS;

	OsziConfig.TrigLV = i32 - OsziConfig.TrigToll;
	if (OsziConfig.TrigLV < TRIG_MAX_NEG) OsziConfig.TrigLV = TRIG_MAX_NEG;

#ifndef INIMP_10K
  TFlashData *fd;
	fd = (void*)FlashBuffer;
	tf = (float)fd->OffsAC / (OsziConfig.dac_digit_V * 10000000.0);
	OsziConfig.AC_Offset = tf;
// printf("AC_Offset=%ld\r\n", OsziConfig.AC_Offset);
#endif

	Oszi_KorrTempAdcOffset();
}

void Oszi_ConfigInit() {
  TFlashData *fd;

	fd = (void*)FlashBuffer;

#ifdef INIMP_10K
	chkf(&fd->V5000mV_Mul, 0.1, 10.0, 1.0);
	chkf(&fd->V2000mV_Mul, 0.1, 10.0, 1.0);
#endif
	chkf(&fd->V1000mV_Mul, 0.1, 10.0, 1.0);
	chkf(&fd->V500mV_Mul, 0.1, 10.0, 1.0);
	chkf(&fd->V200mV_Mul, 0.1, 10.0, 1.0);
	chkf(&fd->V100mV_Mul, 0.1, 10.0, 1.0);
	chkf(&fd->V50mV_Mul, 0.1, 10.0, 1.0);

#ifndef INIMP_10K
/*
 40.2 || 89.8 || 240 = 24.88915 Ohm
 Gain = (360 / 24.88915)+1 = 15.464133
 SollGain 0.02V = 25x
 Vmul = 25 / 15.464133 = 1.616644
*/
//	fd->V20mV_Mul    = 1.616644;
//	chkf(&fd->V20mV_Mul, 0.1, 10.0, 1.616644);
	chkf(&fd->V20mV_Mul, 0.1, 10.0, 1.0);
#endif

#ifdef INIMP_10K
	OsziConfig.tab_idx_V_Div = 3;
#else
	OsziConfig.tab_idx_V_Div = 1;
#endif
	OsziConfig.DataCount = 2500;
//	OsziConfig.InCoup;		/* 0=DC 1=AC */
	OsziConfig.TrigFkt = 0;
	OsziConfig.T_DivCount = 10;
	OsziConfig.BandWidth = OSZI_BANDWIDTH_AUTO;

	OsziConfig.TrigToll = 128;

	OsziConfig.T_Div = 0.0005;
	OsziConfig.TrigTimeProz = 0.5;
	OsziConfig.Samps_pSec = 500000.0;
	OsziConfig.OverSampF = 1.0;
	RingBuf3B_Init(&rb3b, OsziConfig.DataCount);

//	INP_DC_ON;
	OsziConfig.InCoup = 1;

	Ads127l01_SetBandWidth(OsziConfig.BandWidth != 0);

	OsziConfig.dac_Vmul = 1.0;
	Oszi_SetV_Div();
	set_trig_source();

	Oszi_ExtTrigSetV(0.5);

	Ads127l01_Start();
}

/*
#define STATIC_ASSERT(condition) typedef char __CONCAT(_static_assert_, __LINE__)[ (condition) ? 1 : -1]
STATIC_ASSERT((128 - sizeof(struct usb_tmc_bulk_header) - sizeof(TOsziFileHead)) == 100);
*/

void Oszi_DataHeadInit() {
  uint32_t i, *pui;
  uint32_t val, k;
  uint8_t digit;
  TOsziHead *ph;
  char *pc;
  const char *cpc;
//  float tf;

	pui = (uint32_t*)(TmcTxBuffer + sizeof(struct usb_tmc_bulk_header));
	for (i=16;i>0;i--) *pui++ = 0;

	ph = (TOsziHead*)(TmcTxBuffer + sizeof(struct usb_tmc_bulk_header));
	pc = ph->fh.name;

	cpc = "ALDccpWAVE,#6";
	while ((*pc++ = *cpc++));
	pc--;
	val = rb3b.items * 3;
	val += 128 - sizeof(struct usb_tmc_bulk_header) - sizeof(ph->fh.name);
	k = 100000;
	while (k > 6) {
	    digit = val / k;
	    *pc = digit | 0x30; pc++;
	    val -= digit * k;
	    k /= 10;
	}
	*pc = val | 0x30; pc++;
	*pc = 0;

	ph->fh.head_size = 128 - sizeof(struct usb_tmc_bulk_header) - sizeof(TOsziFileHead) ;

	ph->oh.struct_size = sizeof(TOsziDataHead);
	ph->oh.DataCount = rb3b.items;

#ifdef INIMP_10K
#ifdef INIMP_2K5
	cpc = LAY_SNR" 2k5";
#else
	cpc = LAY_SNR" 10k";
#endif
#else
	cpc = LAY_SNR" 50";
#endif
	pc = ph->oh.name;
	while ((*pc++ = *cpc++));

	ph->oh.version = 0;
	ph->oh.dtype = 2;
	ph->oh.samp_bit = 24;
	ph->oh.Channel = 1;
	ph->oh.V_DivCount = OsziConfig.V_DivCount;
	ph->oh.T_DivCount = OsziConfig.T_DivCount;
/*
	if (rb3b.rstat & RB3RST_HAS_DATA) {
	    ph->oh.Data0_Offset = OsziConfig.Data0_Offset;
	} else {
	    ph->oh.Data0_Offset = 0;
	}
*/
	ph->oh.Data0_Offset = rb3b.Data_Offset;
	ph->oh.FrameNum = rb3b.FrameNum;

	ph->oh.GainMul =  OsziConfig.dac_digit_V;

//	tf = 1.0;
//	if (ex_adc1.OverSamp > 0) tf = ex_adc1.OverSamp;

	ph->oh.samp_t =  OsziConfig.OverSampF / OsziConfig.Samps_pSec;

	ph->oh.start_t = -ph->oh.samp_t * OsziConfig.TrigTimeProz *
			 (float)(rb3b.items - 1);

	ph->oh.V_Div = OsziConfig.V_Div;
	ph->oh.T_Div = OsziConfig.T_Div;
}

void Oszi_KorrTempAdcOffset() {
	rb3b.Akt_Offset = OsziConfig.GainOffset + OsziConfig.TempOffset;
#ifndef INIMP_10K
	if (OsziConfig.InCoup == 1) {
	    rb3b.Akt_Offset += OsziConfig.AC_Offset;
	}
#endif
	Ads127l01_SetOffset(rb3b.Akt_Offset);
}

STATIC bool Oszi_SetSampMulDiv(int32_t ovs) {
  int32_t OverSampDiv1, OverSampDiv2;
  uint32_t irrB;

	OverSampDiv1 = 0;
	OverSampDiv2 = 0;
	irrB = 0;

	if (ovs > 1) {
	    if (ovs <= 256) {
		OverSampDiv1 = ovs;
	    } else {
		switch (ovs) {
		case 400:
		    OverSampDiv1 = 200;
		    OverSampDiv2 = 2;
		    break;
		case 500:
		    OverSampDiv1 = 100;
		    OverSampDiv2 = 5;
		    break;
		case 1000:
		    OverSampDiv1 = 200;
		    OverSampDiv2 = 5;
		    break;
		case 2000:
		    OverSampDiv1 = 200;
		    OverSampDiv2 = 10;
		    break;
		case 4000:
		    OverSampDiv1 = 200;
		    OverSampDiv2 = 20;
		    break;
		case 5000:
		    OverSampDiv1 = 250;
		    OverSampDiv2 = 20;
		    break;
		case 10000:
		    OverSampDiv1 = 250;
		    OverSampDiv2 = 40;
		    break;
		case 20000:
		    OverSampDiv1 = 250;
		    OverSampDiv2 = 80;
		    break;
		case 40000:
		    OverSampDiv1 = 250;
		    OverSampDiv2 = 160;
		    break;
		default:
#if defined(__x86_64__) || defined(__i386__)
	printf("ERROR: SetSampMulDiv %d\r\n", ovs);
#else
#if DEBUG_CONSOLE_ON
	printf("ERROR: SetSampMulDiv %ld\r\n", ovs);
#endif
#endif
		    return false;
		}
	    }

	    if (OsziConfig.BandWidth == OSZI_BANDWIDTH_AUTO) {
		if ((ovs > 4) && ADC_FILT1_STAT) {
  float df;
		    df = 1.0 / tanf(M_PI / (float)ovs);
/*
#if DEBUG_CONSOLE_ON
  char buf[32];
  floattopc(buf, 1.0 / (1.0+df), 6);
  printf("ovs=%ld A=%s\r\n", ovs, buf);
#endif
*/
		    df = 2147483648.0 / (1.0+df);
		    irrB = df;
		}
	    }
	}


	Ads127l01_SetIRR(irrB, 0);


	if (ovs > 1) OsziConfig.OverSampF = ovs; else OsziConfig.OverSampF = 1.0;
	Ads127l01_SetOverSampDiv(OverSampDiv1, OverSampDiv2);
	return true;
}

static void set_trig_toffs() {
  float so;

	so = (float)(rb3b.items - 1) * OsziConfig.TrigTimeProz;
	rb3b.trig_pos = so + 1.0;
// printf("trig_toffs=%ld  %ld\r\n", rb3b.trig_pos, rb3b.items);
}

static void setup_msiz() {
  uint32_t msiz, ui;
  float tf;

	Ads127l01_Stop();
	tf = OsziConfig.Samps_pSec * OsziConfig.T_Div * 
		(float)OsziConfig.T_DivCount;
	
	if (tf > (float)RINGBUF3S_MAX_COUNT) {
	    msiz = RINGBUF3S_MAX_COUNT;
	} else {
	    msiz = tf + 0.5;
	}
	if (msiz > OsziConfig.DataCount) msiz = OsziConfig.DataCount;
	RingBuf3B_Init(&rb3b, msiz);

	ui = 0;
	tf /= msiz;
	if (tf > 1.5) ui = tf + 0.1;
	Oszi_SetSampMulDiv(ui);

	set_trig_toffs();
	set_trig_source();

	ExecCmdBef |= EXEC_CLR_BUF_DATA;

}

bool Oszi_SetMDataCount(const char *s) {
  int32_t i32;

	i32 = pctoint(s);
	if (i32 < 0) return false;
	OsziConfig.DataCount = i32;
	if (i32 >= 35000) {
	    OsziConfig.T_DivCount = 14;
	} else {
	    OsziConfig.T_DivCount = 10;
	}

	setup_msiz();
	return true;
}

uint32_t Oszi_GetMDataCount() {
	return OsziConfig.DataCount;
}

static float get_1_2_5F(float val) {
  float k;

	if (val < 1E-12) val = 1E-9;
	k = 1.0;
	while (val > 9.999999) {val *= 0.1; k *= 10.0;}
	while (val <= 0.9999999) {val *= 10.0; k *= 0.1;}

	if (val < 1.5) val = 1.0; else
	if (val < 3.75) val = 2.0; else
	if (val < 7.5) val = 5.0; else
	val = 10.0;
	return val * k;
}

bool Oszi_SetT_Div(const char *s) {
  float tf;

	tf = pctofloat(s);
	if (tf < 0.00015) tf = 0.0001;
	if (tf > 20.0) tf = 20.0;

	OsziConfig.T_Div = get_1_2_5F(tf);
	setup_msiz();
	return true;
}

float Oszi_GetT_Div() {
	return OsziConfig.T_Div;
}

bool Oszi_SetV_DivStr(const char *s) {
  float tf;
  int i;

	Ads127l01_Stop();
	tf = pctofloat(s);
	tf = get_1_2_5F(tf);
#ifdef INIMP_10K
	if (tf > 3.5) {
	    i = 0;
	} else if (tf > 1.5) {
	    i = 1;
	} else if (tf > 0.75) {
	    i = 2;
	} else if (tf > 0.35) {
	    i = 3;
	} else if (tf > 0.15) {
	    i = 4;
	} else if (tf > 0.075) {
	    i = 5;
	} else {
	    i = 6;
	}
#else
	if (tf > 0.75) {
	    i = 0;
	} else if (tf > 0.35) {
	    i = 1;
	} else if (tf > 0.15) {
	    i = 2;
	} else if (tf > 0.075) {
	    i = 3;
	} else if (tf > 0.035) {
	    i = 4;
	} else {
	    i = 5;
	}
#endif
	OsziConfig.tab_idx_V_Div = i;

	Oszi_SetV_Div();
	RingBuf3B_Pos0(&rb3b);
	ExecCmdBef |= EXEC_CLR_BUF_DATA;
	return true;
}

bool Oszi_SetV_Div() {
  const TOsziVpDiv *pVpDiv;
  uint16_t u16;

	u16 = OsziConfig.tab_idx_V_Div;
	if (u16 >= VDIV_COUNT) u16 = 0;
	pVpDiv = &Tab_OsziVpDiv[OsziConfig.tab_idx_V_Div];
// printf("%d %d pOffs=0x%08lx\r\n", u16, OsziConfig.tab_idx_V_Div, (uint32_t)pVpDiv->pVmul);
	OsziConfig.V_DivCount = pVpDiv->DivCount;
	OsziConfig.V_Div = pVpDiv->V_Div;
	OsziConfig.GainOffset = *pVpDiv->pOffs;
	OsziConfig.dac_Vmul = *pVpDiv->pVmul;

	u16 = pVpDiv->Kio;
	if (u16 & 0x01) {
	    INP_ATT_ON;
	    INP_CPEN_ON;
	} else {
	    INP_ATT_OFF;
	    if (INP_DC_STAT == 0) INP_CPEN_OFF;
	}
	if (u16 & 0x02) ADC_K2_ON; else ADC_K2_OFF;
	if (u16 & 0x04) ADC_K3_ON; else ADC_K3_OFF;
	if (u16 & 0x08) ADC_K4_ON; else ADC_K4_OFF;

	if (OsziConfig.dac_Vmul < 0.1) OsziConfig.dac_Vmul = 1.0;

	set_adc_max_V();
// printf("%f V_Div=%f    \"%s\"\n", tf, OsziConfig.V_Div,  s);
// printf("%fV %fuV\n", OsziConfig.dac_V, OsziConfig.dac_digit_V * 1000000.0);
	return true;
}

bool Oszi_SetTrigSource(const char *pc) {
  uint32_t fkt;

	fkt = 0;
	while (*pc && (*pc == ' ')) pc++;
	if (CP1('0') || CP3('O','F','F') || CP2('N','O')) {
//	    OsziConfig.TrigFkt &= ~OSZI_TRIGFKT_MASK;
	} else if (CP3('C','1','S')) {
	    fkt = OSZI_TRIGFKT_ON_SINGLE;
	} else if (CP2('C','1')) {
	    fkt = OSZI_TRIGFKT_ON;
	} else if (CP4('F','R','E','E')) {
	    fkt = OSZI_TRIGFKT_FREERUN;
	} else if (CP4('S','I','N','G')) {
	    fkt = OSZI_TRIGFKT_SINGLE;
	} else if (CP3('E','X','T')) {
	    fkt = OSZI_TRIGFKT_EXT | OSZI_TRIGFKT_ON;
	} else return false;

	OsziConfig.TrigFkt &= ~OSZI_TRIGFKT_MASK;
	OsziConfig.TrigFkt |= fkt;

	Oszi_ExtTrigOutEn((fkt & (OSZI_TRIGFKT_ON | OSZI_TRIGFKT_SINGLE)) != 0); 

	setup_msiz();
	return true;
}

void Oszi_GetTrigSource() {
  const char *s;

	switch (OsziConfig.TrigFkt & OSZI_TRIGFKT_MASK) {
	case OSZI_TRIGFKT_ON:
	    s = "C1";
	    break;
	case OSZI_TRIGFKT_SINGLE:
	    s = "SINGLE";
	    break;
	case OSZI_TRIGFKT_ON_SINGLE:
	    s = "C1S";
	    break;
	case OSZI_TRIGFKT_FREERUN:
	    s = "FREERUN";
	    break;
	case (OSZI_TRIGFKT_EXT | OSZI_TRIGFKT_ON):
	    s = "EXT";
	    break;
	default:
	    s = "NONE";
	}

//	if (rb3b.stat & RB3ST_USE_TRIGER) s = "C1"; else
//	if (rb3b.stat & RB3ST_FREE_RUN) s = "FREERUN";
	scpi_set_str(s);
}

bool Oszi_SetTrigTOffset(const char *s) {

	OsziConfig.TrigTimeProz = pctofloat(s);
	setup_msiz();
	return true;
}
/*
void Oszi_GetTrigTOffset() {

}
*/

bool Oszi_SetTrigToll(const char *pc) {
  uint32_t u32;

	u32 = pctoint(pc);
	if (u32 > 4096) u32 = 4096;
	OsziConfig.TrigToll = u32;
	set_adc_max_V();
	setup_msiz();
	return true;
}

void Oszi_GetTrigToll() {
	scpi_set_str_U32(OsziConfig.TrigToll);
}

bool Oszi_SetTrigType(const char *pc) {
	while (*pc && (*pc == ' ')) pc++;
	if (CP3('P','O','S')) {
	    OsziConfig.TrigFkt &= ~OSZI_TRIGFKT_NEG;
//	    ex_adc1.TrigMod = TRIG_LH;

	} else if (CP3('N','E','G')) {
	    OsziConfig.TrigFkt |= OSZI_TRIGFKT_NEG;
//	    ex_adc1.TrigMod = TRIG_HL;

	} else return false;

	setup_msiz();
	return true;
}

bool Oszi_RunSingle() {
//	if ((OsziConfig.TrigFkt & OSZI_TRIGFKT_SINGLE) == 0) return false;
	setup_msiz();
	return true;
}

void Oszi_GetTrigType() {
  const char *s;

	s = "POS";
	if (OsziConfig.TrigFkt & OSZI_TRIGFKT_NEG) s = "NEG";
	scpi_set_str(s);
}

float Oszi_GetV_Div() {
	return OsziConfig.V_Div;
}

bool Oszi_SetTrigV(const char *s) {
  float TrigV;

	TrigV = pctofloat(s);
	if (OsziConfig.TrigFkt & OSZI_TRIGFKT_EXT) {
	    Oszi_ExtTrigSetV(TrigV);

	} else {
	    OsziConfig.TrigV = pctofloat(s);
	    set_adc_max_V();
	    setup_msiz();
	}
	return true;
}

void Oszi_GetTrigV() {
	if (OsziConfig.TrigFkt & OSZI_TRIGFKT_EXT) {
	    scpi_set_str_float(OsziConfig.ExtTrigV, 3);
	} else {
	    scpi_set_str_float(OsziConfig.TrigV, DECIPOINT_EXP | 3);
	}
}

bool Oszi_SetBandWidthLimit(const char *pc) {
	while (*pc && (*pc == ' ')) pc++;

	if (CP1('0') || CP3('O','F','F')) {
	    OsziConfig.BandWidth = OSZI_BANDWIDTH_OFF;
	    Ads127l01_SetBandWidth(false);
	    setup_msiz();
	    return true;
	}
	if (CP1('1') || CP4('A','U','T','O')) {
	    OsziConfig.BandWidth = OSZI_BANDWIDTH_AUTO;
	    Ads127l01_SetBandWidth(true);
	    setup_msiz();
	    return true;
	}
	return false;
}

bool Oszi_SetInCoup(const char *pc) {
	while (*pc && (*pc == ' ')) pc++;

	if CP2('A','C') {
	    INP_DC_OFF;
	    if (INP_ATT_STAT == 0) INP_CPEN_OFF;
	    OsziConfig.InCoup = 1;
	    return true;
	}
	if CP2('D','C') {
	    INP_DC_ON;
	    INP_CPEN_ON;
	    OsziConfig.InCoup = 0;
	    return true;
	}
	return false;
}

bool Oszi_SetWideBand(const char *pc) {
	while (*pc && (*pc == ' ')) pc++;
	if (CP1('0') || CP3('O','F','F')) {
	    ADC_FILT1_H;
	    setup_msiz();
	    return true;
	}
	if (CP1('1') || CP2('O','N')) {
	    ADC_FILT1_L;
	    setup_msiz();
	    return true;
	}
	return false;
}

static int32_t mIRR_X, mIRR_Y, mOVS_Count, mAnlValSum;

void Oszi_DmmV_Conv(int32_t AnlVal) {
  int32_t zw, res;

	  zw = AnlVal * 4;
	  res = zw + mIRR_X;
	  mIRR_X = zw;
	  zw = (int32_t)((int64_t)((int64_t)mIRR_Y * 31) >> 5);
	  mIRR_Y = res + zw;
	  mAnlValSum += mIRR_Y >> 8;

	  mOVS_Count++;
	  if (mOVS_Count >= 100) {
	      zw = mAnlValSum / mOVS_Count;
	      mOVS_Count = 0;
	      mAnlValSum = 0;
//	      zw = zw - (OsziConfig.GainOffset + OsziConfig.TempOffset);
	      DmmValue = (float)zw * OsziConfig.dac_digit_V;
	  }
}

float Oszi_Get_DmmV() {
	return DmmValue;
}


bool Oszi_TestVal(const char *pc) {
  int32_t i32;

	i32 = pctoint(pc);
	if (i32 < 0) i32 = 0;

	if (i32 > 0) {
	    if (i32 > 16) i32 = 16;
	    i32--;
	    i32 = 0x40000000 >> i32;
	}
	Ads127l01_SetIRR(i32, 0);
/*
	ex_adc1.IRR_InMul = 0;
	i32 = pctoint(pc);
	if (i32 > 16) i32 = 16;
	if (i32 > 0) {
	    i32--;
	    ex_adc1.IRR_InMul = 0x40000000 >> i32;
	    ex_adc1.IRR_yMUL = (uint32_t)0x80000000 - 
			       (uint32_t)(ex_adc1.IRR_InMul << 1);
	}
*/
#if defined(__x86_64__) || defined(__i386__)
 printf("irr=%d 0x%08x\n", ex_adc1.IRR_InMul, ex_adc1.IRR_yMUL);
//#else
// printf("irr=%ld 0x%08x\r\n", ex_adc1.OverSamp, ex_adc1.OverSampShift, ex_adc1.OverSampDiv);
#endif

	return true;
}

bool Oszi_Test_IRR_Val(const char *pc) {
  int32_t i32;

	i32 = pctoint(pc);
	if (i32 < 0) i32 = 0;
	if (i32 >= 0x40000000) i32 = 0;
	Ads127l01_SetIRR(i32, 0);
	return true;
}

int32_t Oszi_GetBufferStat() {
	if (rb3b.stat & RB3ST_FREE_RUN) return 1;
	if (rb3b.rstat & RB3RST_NEW_DATA) return 1;
	return 0;
}

void Oszi_ExtTrigOutEn(bool en) {
  uint32_t m;

	m = GPIOA->MODER & ~GPIO_MODER_MODE3_Msk;
	if (en) m |= GPIO_MODER_MODE3_0;
	GPIOA->MODER = m;
	TRG_IO_L;
	if (en) TRG_O_EN_H; else TRG_O_EN_L;
}

void Oszi_ExtTrigSetV(float v) {
  uint32_t ub12;

	if (v < 0.0) v = 0.0;
	if (v > 4.0) v = 4.0;
	OsziConfig.ExtTrigV = v;
 /* 1 / ((1k8 / 8k2) + 1) = 0.82 * 4096 / 3.3 = 1017.8 */
 /* digit = 3.3 / 4096 */
	v *= 1017.8;
	ub12 = v;
	if (ub12 >= 0x0400) ub12 = 0x03FF;
	DAC1->DHR12R1 = ub12;
}
