
#include "main.h"
#include "usbd_tmc_interface.h"
#include "scpi.h"
#include "utils.h"
#include "oszi.h"
#include "libh523.h"

#define CP_UNALIGN 1

#define END(X) (!pc[X] || (pc[X] == ' '))

#define CP1(a) (pc[0] == a)
#if CP_UNALIGN
#define CP2(a,b) (*(int16_t*)pc == (int16_t)(a|(b << 8)))
//#define CP2(a,b) ((pc[0] == a)&&(pc[1] == b))
#define CP3(a,b,c) (CP2(a,b)&&(pc[2] == c))
#define CP4(a,b,c,d) (*(int32_t*)pc == (int32_t)(a|(b << 8)|(c << 16)|(d << 24)))
#define CP5(a,b,c,d,e) (CP4(a,b,c,d)&&(pc[4] == e))
#else
#define CP2(a,b) ((pc[0] == a)&&(pc[1] == b))
#define CP3(a,b,c) ((pc[0] == a)&&(pc[1] == b)&&(pc[2] == c))
#define CP4(a,b,c,d) ((pc[0] == a)&&(pc[1] == b)&&(pc[2] == c)&&(pc[3] == d))
#define CP5(a,b,c,d,e) ((pc[0] == a)&&(pc[1] == b)&&(pc[2] == c)&&(pc[3] == d)&&(pc[4] == e))
#endif

__attribute__((aligned(4))) 
TScpi Scpi;

void scpi_set_str(const char *s) {
  char *dest;
  const char *src;

	dest = (char*)Scpi.tx_data;
	src = s;
	while ((*dest++ = *src++));
	dest--;
	Scpi.tx_data = (uint8_t*)dest;
}
/*
static void scpi_set_str_i16(int16_t val) {
  char *dest;
	dest = (char*)Scpi.tx_data;
	Scpi.tx_data += Int16ToStr(dest, val);
}
*/
void scpi_set_str_U32(uint32_t val) {
  char *dest;
	dest = (char*)Scpi.tx_data;
	Scpi.tx_data += Uint32ToStr(dest, val);
}

void scpi_set_str_float(float val, int decipoint) {
  char *dest;
	dest = (char*)Scpi.tx_data;
	floattoppc(&dest, val, decipoint);
	Scpi.tx_data = (uint8_t*)dest;
}

static void scpi_set_str_float_exp(float val) {
	scpi_set_str_float(val, DECIPOINT_EXP);
}

static void set_txd_err(TScpi *Scpi) {
  char *dest;
  const char *cpc;
  int len;

	if (Scpi->err[0] == 0) {
	    scpi_set_str("0,No error");
	    return;

	} else {
//  -113,Undefined header;
	    dest = (char*)Scpi->tx_data;
	    *dest = '-'; dest++;
	    len = Uint16ToStr(dest, Scpi->err[0]);
	    dest += len;
	    *dest = ','; dest++;
	    *dest = '\"'; dest++;
	    switch (Scpi->err[0]) {
	    case ERR_SYNTAX:
		cpc = "Syntax error";
		break;
	    case ERR_MISSING_PARAMETER:
		cpc = "Missing parameter";
		break;
	    case ERR_UNDEFINED_HEADER:
		cpc = "Undefined header";
		break;
	    case ERR_ILLEGAL_PARAMETER_VALUE:
		cpc = "Illegal parameter value";
		break;
	    case ERR_OVERVOLTAGE:
		cpc = "Overvoltage";
		break;

	    default:
		cpc = "Undefined";
	    }

//	    dest += MoveStr(dest, cpc);
	    while ((*dest++ = *cpc++));

	    *dest = '\"'; dest++;
	    *dest = 0;
	    Scpi->tx_data = (uint8_t*)dest;
	}
	Scpi->err[0] = Scpi->err[1];
	Scpi->err[1] = Scpi->err[2];
	Scpi->err[2] = Scpi->err[3];
	Scpi->err[3] = 0;
// TEST_PIN2_OFF;
}

static void add_err(TScpi *Scpi, uint16_t err_no) {
	Scpi->err[3] = Scpi->err[2];
	Scpi->err[2] = Scpi->err[1];
	Scpi->err[1] = Scpi->err[0];
	Scpi->err[0] = err_no;
	Scpi->stat &= ~SCPI_OK;
}

void AddError(uint16_t err_no) {
	add_err(&Scpi, err_no);
}

void scpi_line_set(uint8_t *line, uint32_t len, TScpi *Scpi) {
  int i;
  uint8_t *pc;
  int32_t i32;
  float tf, tf2;
  uint16_t err_no;

	err_no = ERR_UNDEFINED_HEADER;

	Scpi->stat = 0;
	Scpi->tx_data = TmcTxBuffer + sizeof(struct usb_tmc_bulk_header);
	Scpi->tx_data_len = 0;

	if (!len) return;
	pc = line;
	if (*pc == ':') pc++;
	while (*pc && *pc != '"') {
	    if ((*pc >= 'a') && (*pc <= 'z')) *pc -= 0x20;  
	    pc++;
	}

	pc = line;
	if (*pc == ':') pc++;
// printf("line=%s\r\n", pc);


	Scpi->stat = SCPI_OK;
	switch (*pc) {
	case '*':
	    pc++;
	    switch (*pc) {

	    case 'C':
		pc++;
		if (CP2('L','S') && END(2)) {	// *CLS CLear Status
		    Scpi->err[0] = 0;
		    Scpi->err[1] = 0;
		    Scpi->err[2] = 0;
		    Scpi->err[3] = 0;
		    return;
		}
		break;

	    case 'E':
		pc++;
		if CP2('S','E') {		// *ESE uint8_t
		    if (pc[2] == '?') {
			scpi_set_str("0");
			return;
		    } else if END(3) return;

		} else if CP3('S','R','?') {	// *ESR uint8_t
		    scpi_set_str("0");
		    return;
		}
		break;

	    case 'I':
		pc++;
		if CP3('D','N','?') {		// *IDN identification string
#ifdef INIMP_10K
#ifdef INIMP_2K5
		    scpi_set_str("ALD Oszi24b 2k5");
#else
		    scpi_set_str("ALD Oszi24b 10k");
#endif
#else
		    scpi_set_str("ALD Oszi24b 50");
#endif
		    return;
		}
		break;

	    case 'O':
		pc++;
		if CP2('P','C') {		// *OPC Operation Complete
		    if (pc[2] == '?') {
			scpi_set_str("1"); /* always completed */
			return;
		    } else if END(2) return;
		}
		break;

	    case 'R':
		pc++;
		if (CP2('S','T') && END(2)) {	// *RST set default settings
		    return;
		}
		break;

	    case 'S':
		pc++;
		if CP2('R','E') {		// *SRE uint8_t
		    if (pc[2] == '?') {
			scpi_set_str("0");
			return;
		    } else if END(2) return;

		} else if CP3('T','B','?') {	// *STB uint8_t status byte
		    scpi_set_str("0");
		    return;
		}
		break;

	    case 'W':
		pc++;
		if (CP2('A','I') && END(2)) {
		    return;
		}
		break;
	    }
	    break;
/* ---------------- ASCII A-Z ------------------------------------------- */

	case 'B':
	    pc++;
	    if CP5('U','F','S','T','?') {
		i32 = Oszi_GetBufferStat();
		if (i32 & 1) {
		    scpi_set_str("1");
		} else {
		    scpi_set_str("0");
		}
		return;
	    }

	case 'C':
	    pc++;
	    if CP2('1', ':') {
		pc += 2;
		if CP3('B','W','L') {
		    pc += 3;
		    if CP1(' ') {			// C1:BWL [0,OFF,AUTO]
			if (Oszi_SetBandWidthLimit((char*)pc)) return;

		    } else if CP1('?') {		// C1:BWL?
			if (OsziConfig.BandWidth) {
			    scpi_set_str("AUTO");
			} else {
			    scpi_set_str("OFF");
			}
			return;
		    }

		} else if CP4('C','O','U','P') {
		    pc += 4;
		    if CP1(' ') {			// C1:COUP [DC,AC]
			if (Oszi_SetInCoup((char*)pc)) return;

		    } else if CP1('?') {		// C1:COUP?
			if (OsziConfig.InCoup == 1) {
			    scpi_set_str("AC");
			} else {
			    scpi_set_str("DC");
			}
			return;
		    }

		} else if CP5('I','N','D','C','?') {	// C1:INDC?
		    scpi_set_str_float(InDC, 3);
		    return;

		} else if CP4('S','T','A','T') {	// C1:STAT [ON,OFF]
		    pc += 4;
		    if CP1(' ') {
			return;
		    } else if CP1('?') {		// C1:STAT?
			scpi_set_str("ON");
			return;
		    }
		} else if CP4('V','D','I','V') {	// C1:VDIV
		    pc += 4;
		    if CP1(' ') {
			if (Oszi_SetV_DivStr((char*)pc)) return;

		    } else if CP1('?') {		// C1:VDIV?
			tf = Oszi_GetV_Div();
			scpi_set_str_float_exp(tf);
			return;
		    }
		} else if CP5('W','I','D','E','B') {	// C1:WIDEB
		    pc += 5;
		    if CP1(' ') {
			if (Oszi_SetWideBand((char*)pc)) return;

		    } else if CP1('?') {		// C1:WIDEB?
			if (ADC_FILT1_STAT) {
			    scpi_set_str("OFF");
			} else {
			    scpi_set_str("ON");
			}
			return;
		    }
		}

	    } else if CP3('A','L',':') {		// CAL:
		pc += 3;

		if CP2('G','V') {
		    float *pf = NULL;
		    TFlashData *fd = (void*)FlashBuffer;

		    pc += 2;
		    if CP4('5','0','0','M') {		// CAL:GV500m
			pc += 4;
			pf = &fd->V500mV_Mul;
		    } else if CP4('2','0','0','M') {
			pc += 4;
			pf = &fd->V200mV_Mul;
		    } else if CP4('1','0','0','M') {
			pc += 4;
			pf = &fd->V100mV_Mul;
		    } else if CP3('5','0','M') {
			pc += 3;
			pf = &fd->V50mV_Mul;
#ifndef INIMP_10K
		    } else if CP3('2','0','M') {
			pc += 3;
			pf = &fd->V20mV_Mul;
#endif
		    } else if CP5('1','0','0','0','M') {
			pc += 5;
			pf = &fd->V1000mV_Mul;
#ifdef INIMP_10K
		    } else if CP5('2','0','0','0','M') {
			pc += 5;
			pf = &fd->V2000mV_Mul;
		    } else if CP5('5','0','0','0','M') {
			pc += 5;
			pf = &fd->V5000mV_Mul;
#endif
		    }

		    if (pf) {
			if CP1(' ') {
			    tf = pctofloat((char*)pc);
			    if ((tf > 0.1) && (tf < 10.0)) {
#ifndef SIM_PC
	__ASM ("str	%1, [%0, #0]" : : __CMSIS_GCC_USE_REG (pf), __CMSIS_GCC_USE_REG (tf) );
#else
				*pf = tf;
#endif
				Oszi_SetV_Div();
				return;
			    }
			    err_no = ERR_ILLEGAL_PARAMETER_VALUE;

			} else if CP1('?') {		// CAL:GV500M?
#ifndef SIM_PC
  __ASM ("ldr	%0, [%1, #0]" : __CMSIS_GCC_OUT_REG (tf) : __CMSIS_GCC_USE_REG (pf) );
			    scpi_set_str_float(tf, 6);
			    return;
#else
/* HardFault zB.: Value = 0xFFFFFFFF !!!!!!! ????? !!!!!!! */
			    scpi_set_str_float(*pf, 6);
			    return;
#endif
			}
		    }

		} else if CP5('I','N','D','C',':') {	// CAL:INDC:
		    pc += 5;
		    if CP4('D','I','G','V') {		// CAL:INDC:DIGV
			pc += 4;
			if CP1(' ') {
			    pc++;
			    tf = pctofloat((char*)pc);
			    if ((tf >= 0.0001) && (tf < 0.065536)) {
				InDC_DigitV = tf;
				TFlashData *fd = (void*)FlashBuffer;
				fd->InDC_DigitV = tf * 1000000.0;
				return;
			    }
			    err_no = ERR_ILLEGAL_PARAMETER_VALUE;

			} else if CP1('?') {		// CAL:INDC:DIGV?
			    TFlashData *fd = (void*)FlashBuffer;
			    tf = (float)fd->InDC_DigitV * 0.000001;
			    scpi_set_str_float(tf, 6);
			    return;
			}

		    } else if CP3('O','F','F') {	// CAL:INDC:OFF
			TFlashData *fd = (void*)FlashBuffer;
			tf2 = INDC_DIGIT_V;
			if ((fd->InDC_DigitV >= 100) && (fd->InDC_DigitV != 0xFFFF)) {
			    tf2 = (float)fd->InDC_DigitV * 0.000001;
			}

			pc += 3;
			if CP1(' ') {
			    pc++;
			    tf = pctofloat((char*)pc) / tf2;
			    if (fabsf(tf) < 32767) {
				fd->InDC_Offset = tf;
				return;
			    }
			    err_no = ERR_ILLEGAL_PARAMETER_VALUE;

			} else if CP1('?') {		// CAL:INDC:OFF?
			    tf = (float)fd->InDC_Offset * tf2;
			    scpi_set_str_float(tf, 3);
			    return;
			}
		    }

		} if CP2('O','V') {	// Offset Digit
		    TFlashData *fd = (void*)FlashBuffer;
		    float gv = 0;
		    int16_t *pi16 = NULL;

		    pc += 2;
		    if CP4('5','0','0','M') {		// CAL:OV500m
			pc += 4;
			gv = fd->V500mV_Mul * 5.0;
			pi16 = &fd->V500mV_Offs;
		    } else if CP4('2','0','0','M') {
			pc += 4;
			gv = fd->V200mV_Mul * 2.0;
			pi16 = &fd->V200mV_Offs;
		    } else if CP4('1','0','0','M') {
			pc += 4;
			gv = fd->V100mV_Mul;
			pi16 = &fd->V100mV_Offs;
		    } else if CP3('5','0','M') {
			pc += 3;
			gv = fd->V50mV_Mul * 0.5;
			pi16 = &fd->V50mV_Offs;
#ifndef INIMP_10K
		    } else if CP3('2','0','M') {
			pc += 3;
			gv = fd->V20mV_Mul * 0.2 * 1.6;
			pi16 = &fd->V20mV_Offs;
#endif
		    } else if CP5('1','0','0','0','M') {
			pc += 5;
			gv = fd->V1000mV_Mul * 10.0;
			pi16 = &fd->V1000mV_Offs;
#ifdef INIMP_10K
		    } else if CP5('2','0','0','0','M') {
			pc += 5;
			gv = fd->V2000mV_Mul * 20.0;
			pi16 = &fd->V2000mV_Offs;
		    } else if CP5('5','0','0','0','M') {
			pc += 5;
			gv = fd->V5000mV_Mul * 50.0;
			pi16 = &fd->V5000mV_Offs;
#endif
		    }

		    if (pi16) {
		        gv *= ADC_BASE_KORR;
			if CP1(' ') {		// CAL:GV500 1E-6
			    tf = pctofloat((char*)pc);
			    tf /= gv;
			    if ((tf > -32768.0) && (tf < 32767.0)) {
#if __ARM_ARCH >= 8 && (__ARM_FP & 0x4) && !defined (__SOFTFP__)
				asm volatile ("vrinta.f32\t%0, %0" : : "t" (tf));
#else
				if (tf > 0.0) tf += 0.5; else tf -= 0.5;
#endif
				int16_t i16 = tf;
				*pi16 = i16;
				Oszi_SetV_Div();
				return;
			    }
			    err_no = ERR_ILLEGAL_PARAMETER_VALUE;

			} else if CP1('?') {		// CAL:GV500M?
			    tf = *pi16;
			    tf *= gv;
			    scpi_set_str_float(tf, DECIPOINT_EXP | 3);
			    return;
			}
		    }

		} if CP3('T','V','K') {	// Relais On/Off
		    TFlashData *fd = (void*)FlashBuffer;
		    float gv = 0;
		    int16_t *pi16 = NULL;

		    pc += 3;
		    if CP1('2') {
			pc += 1;
			gv = fd->V200mV_Mul * 2.0;
			pi16 = &fd->K2_TV;
		    } else if CP1('3') {
			pc += 1;
			gv = fd->V100mV_Mul;
			pi16 = &fd->K3_TV;
		    } else if CP1('4') {
			pc += 1;
			gv = fd->V50mV_Mul * 0.5;
			pi16 = &fd->K4_TV;
		    }

		    if (pi16) {
		        gv *= ADC_BASE_KORR;
			if CP1(' ') {		// CAL:TV500 1E-6
			    tf = pctofloat((char*)pc);
			    tf /= gv;
			    if ((tf > -32768.0) && (tf < 32767.0)) {
#if __ARM_ARCH >= 8 && (__ARM_FP & 0x4) && !defined (__SOFTFP__)
				asm volatile ("vrinta.f32\t%0, %0" : : "t" (tf));
#else
				if (tf > 0.0) tf += 0.5; else tf -= 0.5;
#endif
				int16_t i16 = tf;
				*pi16 = i16;
				Oszi_SetV_Div();
				return;
			    }
			    err_no = ERR_ILLEGAL_PARAMETER_VALUE;

			} else if CP1('?') {		// CAL:TV500M?
			    tf = *pi16;
			    tf *= gv;
			    scpi_set_str_float(tf, DECIPOINT_EXP | 3);
			    return;
			}
		    }

		} else if CP5('T','E','M','P','O') {
		    pc += 5;
		    if CP1(' ') {			// CAL:TEMPO 0.1C
			tf = pctofloat((char*)pc);
			if ((tf > -30.01) && (tf < 30.01)) {
			    TFlashData *fd = (void*)FlashBuffer;
			    fd->TempOffs = tf;
			    TMP236SetTempOffset(tf);
			    return;
			}
			err_no = ERR_ILLEGAL_PARAMETER_VALUE;
		    } else if CP1('?') {		// CAL:TEMPO?
			TFlashData *fd = (void*)FlashBuffer;
			scpi_set_str_float(fd->TempOffs, 1);
			return;
		    }

		} else if CP5('T','E','M','P','K') {
		    pc += 5;
		    if CP1(' ') {			// CAL:TEMPK 1E-7 (0.1uV/C)
			TFlashData *fd = (void*)FlashBuffer;
			tf = pctofloat((char*)pc);
			if ((tf > -0.001) && (tf < 0.001)) {
			    fd->AdcVpC = tf;
			    return;
			}
			err_no = ERR_ILLEGAL_PARAMETER_VALUE;
		    } else if CP1('?') {		// CAL:TEMPK?
			TFlashData *fd = (void*)FlashBuffer;
			scpi_set_str_float(fd->AdcVpC, DECIPOINT_EXP | 3);
			return;
		    }

		} else if (CP5('T','E','M','P','?') && END(5)) { // CAL:TEMP?
		    tf = TMP236GetTemp(anl0_val.u);
		    scpi_set_str_float(tf, 1);
//		    scpi_set_str_i16(i);
		    return;


		} else if (CP4('S','A','V','E') && END(4)) {
		    ExecCmdBef |= EXEC_SAVE_CAL_DATA;	// CAL:SAVE
		    return;

#if USE_OSZI_TDIV_MOD == 0
		} else if CP4('O','S','R',' ') {
		    pc += 3;
		    if (Oszi_CalOSR((char*)pc)) return;

		} else if CP5('O','S','A','P',' ') {
		    pc += 5;
		    if (Oszi_CalOverSamp((char*)pc)) return;
#endif /* USE_OSZI_TDIV_MOD 0 */
		}
	    }


	    break;

	case 'D':
	    pc++;
	    if CP4('A','T','A','?') {
		RingBuf3B_GetDataInit(&rb3b);
		Oszi_DataHeadInit();
		Scpi->stat |= SCPI_TXD_BUF;
		Scpi->tx_data += 116 + (rb3b.items * 3);
		return;
	    } else if CP3('M','M','?') {
		i = 6;
		tf = Oszi_GetV_Div();
		if (tf < 0.211) i = 7;
		tf = Oszi_Get_DmmV();
		scpi_set_str_float(tf, i);
		return;

	    } else if CP4('M','M','+','?') {
		i = 6;
		tf = Oszi_GetV_Div();
		if (tf < 0.211) i = 7;
		tf = Oszi_Get_DmmV();
		scpi_set_str_float(tf, i);
		scpi_set_str("V,");
		tf = TMP236GetTemp(anl0_val.u);
		scpi_set_str_float(tf, 1);
		scpi_set_str("C,");
		tf = Oszi_GetV_Div();
		scpi_set_str_float(tf, 3);
		scpi_set_str("V/Div");
		return;
	    }
	    break;

	case 'M':
	    pc++;
	    if CP3('S','I','Z') {
		pc += 3;
		if CP1(' ') {
		    if (Oszi_SetMDataCount((char*)pc)) return;
		    add_err(Scpi, ERR_SYNTAX);

		} else if CP1('?') {
		    i32 = Oszi_GetMDataCount(); //RingBuf3B_GetCount(&rb3b);
		    scpi_set_str_U32(i32);
		    return;
		}
	    }
	    break;

	case 'R':
	    pc++;
	    if CP3('U','N','S') {		//RUNSingle
		if (Oszi_RunSingle()) return;
	    }

	case 'S':
	    pc++;
	    if CP3('Y','S','T') {
		pc += 3;
		if CP2('E','M') pc += 2;
		if CP1(':') {
		    pc++;
		    if CP3('E','R','R') {
			pc += 3;
			if CP2('O','R') pc += 2;
			if CP1('?') {		// SYSTem:ERRor?
			    set_txd_err(Scpi);
			    return;
			}
		    }
		}
	    }
	    break;

	case 'T':
	    pc++;
	    if CP3('D','I','V') {
		pc += 3;
		if CP1(' ') {			// TDIV 
		    if (Oszi_SetT_Div((char*)pc)) return;

		} else if CP1('?') {		// TDIV?
		    tf = Oszi_GetT_Div();
		    scpi_set_str_float(tf, DECIPOINT_EXP);
		    return;
		}
	    } else if CP4('R','G','H','Y') {
		pc += 4;
		if CP1(' ') {			// TRGHY 0..4096
		    if (Oszi_SetTrigToll((char*)pc)) return;

		} else if CP1('?') {		// TRGHY?
		    Oszi_GetTrigToll();
		    return;
		}
	    } else if CP3('R','G','O') {
		pc += 3;
		if CP1(' ') {			// TRGO [0..100%]
		    if (Oszi_SetTrigTOffset((char*)pc)) return;

		} else if CP1('?') {		// TRGO?
//		    Oszi_GetTrigTOffset();
		    scpi_set_str_float(OsziConfig.TrigTimeProz, 3);
		    return;
		}
	    } else if CP3('R','G','S') {
		pc += 3;
		if CP1(' ') {			// TRGS [OFF,C1,SINGLE,FREERUN,EXT]
		    if (Oszi_SetTrigSource((char*)pc)) return;

		} else if CP1('?') {		// TRGS?
		    Oszi_GetTrigSource();
		    return;
		}
	    } else if CP3('R','G','V') {
		pc += 3;
		if CP1(' ') {			// TRGV 
		    if (Oszi_SetTrigV((char*)pc)) return;

		} else if CP1('?') {		// TRGV?
		    Oszi_GetTrigV();
//		    scpi_set_str_float(OsziConfig.TrigV, DECIPOINT_EXP | 3);
		    return;
		}
	    } else if CP3('R','I','G') {
		pc += 3;
		if CP1(' ') {			// TRIG [POS,NEG]
		    if (Oszi_SetTrigType((char*)pc)) return;

		} else if CP1('?') {		// TRIG?
		    Oszi_GetTrigType();
		    return;
		}
	    }
	    break;

	case 'U':
	    pc++;
	    if CP4('S','B','V','?') {
		scpi_set_str_float(UsbV, 2);
		return;
	    }

	case 'X':
	    pc++;
	    if CP3('T','I',' ') {
		pc += 3;
		if (Oszi_Test_IRR_Val((char*)pc)) return;

	    } else if CP2('T',' ') {
		pc += 2;
		if (Oszi_TestVal((char*)pc)) return;

	    } else if CP3('L','C','?') {
		scpi_set_str_U32(LoopCount);
		return;

	    } else if CP2('A','0') {
		Ads127l01_Stop();
		return;

	    } else if CP2('A','1') {
		Ads127l01_Start();
		return;

	    } else if CP5('T','E','S','T',' ') {
		pc += 5;
		CmdTestBef = pctoint((const char*)pc);
// Txc32 xc32;
//		xc32.u = 0xFFFFFFFF;
//		CmdTestFloat = xc32.f;
		return;
	    }
	}
	add_err(Scpi, err_no);

	return;
}
